#!/usr/bin/env python
# -*- coding: utf-8 -*-
#       gsocket.py
#
#       Copyright 2008 Gord Allott <gordallott@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#
import urllib
import os, sys
from string import Template
import socket
import gobject

import random

class gSocket(socket.socket):
    """
        demonstration class only coded for clarity, not efficiency
    """

    # buf_out is a list of tuples, first item in the tuple is the actual data
    # second item is the total size of the data, third is the current position
    #
    # data can be a string or a file object (read permissions)

    buf_out = []
    buf_size = 4096 #32 * 1024 # 32KB buffer

    buf_in = ""

    watch_out = None
    watch_in = None

    readCallback = None

    bytes_out = 0
    bytes_in = 0

    def __init__(self):

        socket.socket.__init__(self, socket.AF_INET, socket.SOCK_STREAM)

    def connect(self, (host, port)):
        socket.socket.connect(self, (host, port))

        self.setblocking(False)
        #set our watchers
        self.watch_out = gobject.io_add_watch(  self.fileno(),
                                                gobject.IO_OUT,
                                                self.callbackIO_OUT)

        self.watch_in = gobject.io_add_watch(   self.fileno(),
                                                gobject.IO_IN,
                                                self.callbackIO_IN)

        self.watch_fin = gobject.io_add_watch(  self.fileno(),
                                                gobject.IO_HUP,
                                                self.callbackIO_HUP)

        #self.watch_err = gobject.io_add_watch(  self.fileno(),
                                                #gobject.IO_ERR,
                                                #self.callbackIO_ERR)

    def close(self):
        """ closes the socket """
        gobject.source_remove(self.watch_out)
        gobject.source_remove(self.watch_in)
        gobject.source_remove(self.watch_fin)

        try:
            self.setblocking(True)
            self.shutdown(socket.SHUT_RDWR)
            socket.socket.close(self)
        except:
            raise Exception, 'oh feck...'


    def callbackIO_OUT(self, source, condition):
        """ called when data can be written """
        # we build up a buffer of self.buf_size
        # we do this by going to the first item in the self.buf_out and reading
        # either self.buf_size bytes from it or as much as possible if their is
        # less than self.buf_size bytes in the item. if so we proceed to the
        # next item. we then send that buffer

        # if our buffer is nothing we disable our iowatch

        if len(self.buf_out) == 0:
            gobject.source_remove(self.watch_out)
            return False

        buffer = ""
        totalbytes = self.buf_size

        while totalbytes > 0:
            if not len(self.buf_out):
                break

            item, itemSize, itemPos = self.buf_out[0]
            read_bytes = itemSize - itemPos

            if read_bytes > totalbytes:
                read_bytes = totalbytes

            #read from item, start at last position, read for item_bytes bytes
            if type(item) == type(""):
                buffer += item[itemPos:][:read_bytes]
            elif type(item) == file:
                buffer += item.read(read_bytes)

            #increase our item pos
            itemPos = itemPos + read_bytes

            if itemPos >= itemSize:
                # we have reached the end of this item, we should remove it
                if type(item) == file:
                    #we need to close our files
                    item.close()

                self.buf_out.pop(0)

            else:
                # we still have to write this item so write the new position
                # into it
                #self.buf_out[0][2] = itemPos
                self.buf_out[0] = (item, itemSize, itemPos)

            # decrease our totalbytes as we have filled up the buffer somewhat
            totalbytes = totalbytes - read_bytes

        # now we 'send' our buffer finally :)
        if self.sendall(buffer):
            raise Exception, 'some sort of error when sending data'
        self.bytes_out += len(buffer)

        return True

    def callbackIO_IN(self, source, condition):
        """ called when there is data to be read """

        buf = self.recv(self.buf_size)
        self.bytes_in += len(buf)

        self.buf_in += buf

        #print self.buf_in

        if self.readCallback:
            ret = self.readCallback(self.buf_in)
            self.buf_in = ""

            if not ret:
                self.readCallback = None



        return True


    def callbackIO_ERR(self, source, condition):
        """ called when the connection has errors """
        raise Exception, "connection has errors? .... "
        return self.callbackIO_HUP(source, condition)

    def callbackIO_HUP(self, source, condition):
        """ called when the connection died """
        print "connection died!!!!!!!"
        self.close()


        return False


    def writestr(self, msg):
        """ queues up data to be written, accepts strings """
        #print msg
        self.buf_out.append((msg, len(msg), 0))
        if not self.watch_out:
            self.watch_out = gobject.io_add_watch(  self.fileno(),
                                                    gobject.IO_OUT,
                                                    self.callbackIO_OUT)

    def writefile(self, (file, length)):
        """ adds the given file to the queue, length should be the total length
        in bytes of the file
        """
        self.buf_out.append((file, length, 0))

        if not self.watch_out:
            self.watch_out = gobject.io_add_watch(  self.fileno(),
                                                    gobject.IO_OUT,
                                                    self.callbackIO_OUT)

    def readstr(self):
        """ reads everything thats currently in the buffer returns "" """
        buf = self.buf_in
        self.buf_in = ""
        return buf

    def setReadCallback(self, callbackFunc):
        """ sets a callback, the callback will be called with the read buffer
        as the first argument whenever there is data to be read
        it must return True for it to be active again, if it returns false the
        callback will become null and void
        """
        self.readCallback = callbackFunc


teststr = """GET / HTTP/1.1
Host: localhost
user-agent: Foobar

""".replace("\n", "\r\n")

if __name__ == "__main__":
    #just for testing

    oursocket = gSocket()
    oursocket.connect(('localhost', 80))

    oursocket.writestr(teststr)

    def callback_read(buf):
        if not buf:
            oursocket.close()
            sys.exit(0)
            return False

        print buf
        return True

    oursocket.setReadCallback(callback_read)

    gobject.MainLoop().run()
