#!/usr/bin/env python
# -*- coding: utf-8 -*-
#       guiUploader.py
#
#       Copyright 2008 Gordon Allott <gordallott@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#
import sys, os
import gobject
import pygst
pygst.require("0.10")
import gst

#helper objects for use in gstreamer


class FileSource(gst.Bin):

    def __init__(self, location, caps=""):
        """ creates a file source that will link to the given sink"""
        gst.Bin.__init__(self)
        self.caps = caps

        self._videosink = videosink
        self._audiosink = audiosink

        self._queue = gst.element_factory_make("queue")
        self._queue.set_property("max-size-buffers",500)
        self._filesrc = gst.element_factory_make("filesrc", "source")
        self._filesrc.set_property("location", location)

        self._decodebin = gst.element_factory_make("decodebin", "decode")
        self._decodebin.connect("new-decoded-pad", self.new_decoded_pad)
        self._decodebin.connect('no-more-pads', self.no-more-pads)

        self.add(self._filesrc,
                 self._decodebin,
                 self._queue)

        gst.element_link_many(self._filesrc,
                              self._decodebin
                              #self._queue
                              )

        # Create GhostPad
        self.add_pad(gst.GhostPad('src', self._queue.get_pad('src')))

    def new_decoded_pad(self, element, pad, islast):
        """ this is called when we have a new pad, we connect the pad up to our
        queue pad and nothing more :)
        """
        caps = pad.get_caps()
        pad_type = caps.to_string()[0:5]
        if pad_type == "video":
            pad.link(self._queue.get_pad("sink"))

        elif pad_type == "audio":
            pad.link(self._queue.get_pad("sink"))

        else:
            raise Exception, "Unknown pad type detected, %s" % pad_type

    def no_more_pads(self):
        """ called when we are done connecting up, we can now emit our 'done'
        signal (no-more-pads)
        """
        pass
        #self.emit('no-more-pads')

class VideoEncoder(gst.Bin):
    def __init__(self, quality=100.0, videorate=None, framerate=None):
        """
        creates a nice theora encoding chain, quality should be a value
        ranging from 0 (worst) to 100 (best)
        """
        gst.Bin.__init__(self)

        quality = quality if quality < 100.0 else quality
        quality = quality if quality > 0.0 else 0.0
        self.quality = quality * (63.0 / 100.0)

        self._queue1 = gst.element_factory_make("queue", "videoEncoder-inqueue")
        self._queue1.set_property("max-size-buffers",500)
        self._queue2 = gst.element_factory_make("queue", "videoEncoder-outqueue")
        self._queue1.set_property("max-size-buffers",500)
        self._ffmpegcsp = gst.element_factory_make("ffmpegcolorspace")

        self.encoder = gst.element_factory_make("theoraenc", "videoEncoder-encoder")
        self.encoder.set_property('sharpness', 1)
        self.encoder.set_property("quality", int(self.quality))
        self.encoder.set_state(gst.STATE_PAUSED)

        if framerate:
            framerate = int(framerate)
            self._videorate = gst.element_factory_make("videorate", "videoEncoder-videorate")
            self._videocaps = gst.caps_from_string('video/x-raw-yuv,framerate=%i/1' % framerate)
            self.add(self._videorate)
            self._videoratecaps = gst.element_factory_make("capsfilter", "filter")
            self.add(self._videoratecaps)
            self._videoratecaps.set_property("caps", self._videocaps)



        self.add(self._queue1,
                 self._ffmpegcsp,
                 self.encoder,
                 self._queue2)

        gst.element_link_many(self._queue1,
                              self._ffmpegcsp)
        lastlinked = self._ffmpegcsp

        if framerate:
            lastlinked.link(self._videorate)
            self._videorate.link(self._videoratecaps)
            lastlinked = self._videoratecaps

        gst.element_link_many(lastlinked,
                              self.encoder,
                              self._queue2)

        # Create GhostPads
        self.add_pad(gst.GhostPad('sink', self._queue1.get_pad('sink')))
        self.add_pad(gst.GhostPad('src', self._queue2.get_pad('src')))


class VideoEncoderVR(gst.Bin):
    def __init__(self, quality=100.0, videorate=True):
        """
        creates a nice theora encoding chain, quality should be a value
        ranging from 0 (worst) to 100 (best)
        """
        gst.Bin.__init__(self)

        quality = quality if quality < 100.0 else quality
        quality = quality if quality > 0.0 else 0.0
        self.quality = quality * (63.0 / 100.0)

        self._queue1 = gst.element_factory_make("queue", "videoEncoder-inqueue")
        self._queue1.set_property("max-size-buffers",500)
        self._queue2 = gst.element_factory_make("queue", "videoEncoder-outqueue")
        self._queue1.set_property("max-size-buffers",500)
        self._ffmpegcsp = gst.element_factory_make("ffmpegcolorspace")
        if videorate:
            self._videorate = gst.element_factory_make("videorate", "videoEncoder-videorate")

        else:
            self._videorate = gst.element_factory_make("identity", "videoEncoder-identify")

        self.encoder = gst.element_factory_make("theoraenc", "videoEncoder-encoder")
        self.encoder.set_property('sharpness', 1)
        self.encoder.set_property("quality", int(self.quality))

        self.add(self._queue1,
                 self._ffmpegcsp,
                 self._videorate,
                 self.encoder,
                 self._queue2)

        gst.element_link_many(self._queue1,
                              self._ffmpegcsp,
                              self._videorate,
                              self.encoder,
                              self._queue2)

        # Create GhostPads
        self.add_pad(gst.GhostPad('sink', self._queue1.get_pad('sink')))
        self.add_pad(gst.GhostPad('src', self._queue2.get_pad('src')))


class AudioEncoder(gst.Bin):
    def __init__(self, quality=100.0):
        """
        creates a nice vorbis encoding chain, quality should be a value
        ranging from 0 (worst) to 100 (best)
        """
        gst.Bin.__init__(self)

        quality = quality if quality < 100.0 else 100.0
        quality = quality if quality > 0.0 else 0.0
        self.quality = quality * (1.0 / 100.0)

        self._queue1 = gst.element_factory_make("queue", "audioEncoder-inqueue")
        self._queue1.set_property("max-size-buffers",500)
        self._queue2 = gst.element_factory_make("queue", "audioEncoder-outqueue")
        self._queue2.set_property("max-size-buffers",500)
        self._audioconvert = gst.element_factory_make("audioconvert", "audioEncoder-convert")

        # Vorbisenc can't handle packets without timestamps very well.
        # This means that certain demuxers (asfdemux and matroskademux for
        # example, but also most of the ffdemux elements) will generate
        # very choppy audio.
        #
        # Fortunately GStreamer includes an audiorate element which can fix up
        # such broken streams, by putting the right timestamps on and so forth.

        self._audiorate = gst.element_factory_make("audiorate", "audioEncoder-audiorate")


        self.encoder = gst.element_factory_make("vorbisenc", "audioEncoder-encoder")
        self.encoder.set_property("quality", self.quality)

        self.add(self._queue1,
                 self._audioconvert,
                 self._audiorate,
                 self.encoder,
                 self._queue2)

        gst.element_link_many(self._queue1,
                              self._audioconvert,
                              self._audiorate,
                              self.encoder,
                              self._queue2)

        # Create GhostPads
        self.add_pad(gst.GhostPad('sink', self._queue1.get_pad('sink')))
        self.add_pad(gst.GhostPad('src', self._queue2.get_pad('src')))
