#!/usr/bin/env python
# -*- coding: utf-8 -*-
#       guiUploader.py
#
#       Copyright 2008 Gordon Allott <gordallott@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#
import sys, os
import pygtk
pygtk.require('2.0')
import gtk, gtk.glade, gobject
import resource, iconHandler, guiUploadConfig
from resource import extVideo, extAudio, mb_it
import datetime
from math import *
import pygst
pygst.require("0.10")
import gst

import re
from os import path as ospath
from resource import slugify
from time import sleep

import subprocess
from subprocess import PIPE, call, Popen

import gui
import gstHelpers

from math import pow
def cubicInterpolate(v0, v1, v2, v3, x):
    """ performs generic cubic interpolation """
    P = (v3 - v2) - (v0 - v1)
    Q = (v0 - v1) - P
    R = v2 - v0
    S = v1

    return P * pow(x, 3) + Q * pow(x, 2) + R * x + S

#gst-launch-0.10 multifilesrc location="preview_0%03d.png" index=1 \
#caps="image/png,framerate=\(fraction\)25/1" ! pngdec ! ffmpegcolorspace \
#! theoraenc ! oggmux ! filesink location="images.ogg"

class Encoder(gobject.GObject):
    """ class to handle encoding of many files to one ogg vorbis file,
    i think the general idea is that there are many png files then one large
    file that the many png files are 'attached' to
    """
    __gsignals__ = {
            'ready' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
            'complete' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
            'error' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
        }

    display = None

    imagemask = '/tmp/intro_%04d.png'
    video_src = ''
    video_out = ''

    def __init__(self, display):
        """ display is the gtkdrawable that the output is drawn to
        """
        gobject.GObject.__init__(self)
        self.display = display


        self._pipeline = gst.Pipeline("player")

        bus = self._pipeline.get_bus()
        bus.add_signal_watch()
        bus.connect("message", self.on_message)


        self._videosource = gst.element_factory_make("multifilesrc", "source_file_png")
        self._videosource.set_property("location", '/tmp/intro_%04d.png')
        self._videosource.set_property("index", 1)
        self._videocaps = gst.Caps(r'image/png,framerate=(fraction)25/1')
        self._videosource.set_property("caps", self._videocaps)

        self._audiosource = gst.element_factory_make('filesrc')
        self._audiosource.set_property('location', resource.get_resource('introSound.oga'))

        self._videodecoder = gst.element_factory_make("pngdec", "pngdec")

        self._audiodecoder = gst.element_factory_make('decodebin')
        self._audiodecoder.connect('new-decoded-pad', self.on_new_decoded_pad)

        self._videoencoder = gstHelpers.VideoEncoder(videorate=False)
        self._audioencoder = gstHelpers.AudioEncoder()

        self._mux = gst.element_factory_make('oggmux', 'mux')

        self._progreport = gst.element_factory_make('progressreport')

        self._filesink = gst.element_factory_make('filesink', 'filesink')
        self._filesink.set_property('location', '/tmp/intro.ogg')


        self._pipeline.add(self._videosource,
                           self._audiosource,
                           self._audiodecoder,
                           self._videodecoder,
                           self._videoencoder,
                           self._audioencoder,
                           self._mux,
                           self._progreport,
                           self._filesink)


        # we hook up our video decoder/encoders here
        gst.element_link_many(self._videosource,
                              self._videodecoder,
                              self._videoencoder,
                              self._mux)

        # we need to hook up our audio sources, but not the decoder as that
        # gets hooked up after a dynamic pad is called
        gst.element_link_many(self._audiosource, self._audiodecoder)

        # connect up our ogg muxer to our file sink
        gst.element_link_many(self._mux, self._progreport, self._filesink)

        gobject.idle_add(self.ready)



    def on_new_decoded_pad(self, element, pad, islast):
        caps = pad.get_caps()
        pad_type = caps.to_string()[0:5]
        if pad_type == "video":
            print "ignoring video"

        elif pad_type == "audio":
            self._audioencoder.set_state(gst.STATE_PAUSED)
            pad.link(self._audioencoder.get_pad("sink"))
            self._audioencoder.link(self._mux)
            self.ready()
        else:
            print "Unknown pad type detected, %s" %pad_type


    def sync(self):
        """
        Wait for the pipeline to change state
        We give it a 3 second timeout: returns True if the state change was
        successful in this time, False otherwise
        """

        state = self._pipeline.get_state(timeout=3*gst.SECOND)[0]
        if state == gst.STATE_CHANGE_SUCCESS:
            return True
        else:
            return False

    def ready(self):
        """ called to emit the ready signal """
        self.sync()
        self.emit('ready')
        return False


    def play(self):
        self._pipeline.set_state(gst.STATE_PLAYING)

    def pause(self):
        self._pipeline.set_state(gst.STATE_PAUSED)

    def stop(self):
        self.bus.remove_signal_watch()
        self._pipeline.set_state(gst.STATE_NULL)


    def get_position(self):
        try:
            return self._pipeline.query_position(gst.FORMAT_TIME)[0]
        except gst.QueryError:
            return None

    def get_duration(self):
        try:
            return self._pipeline.query_duration(gst.FORMAT_TIME)[0]
        except gst.QueryError:
            return None

    def on_message(self, bus, message):

        type = message.type
        if type == gst.MESSAGE_EOS:
            self._pipeline.set_state(gst.STATE_NULL)
            self.emit('complete')

        elif type == gst.MESSAGE_ERROR:
            self._pipeline.set_state(gst.STATE_NULL)
            err, debug = message.parse_error()
            print "Error: %s" % err, debug
            self.emit('error')






class Render(object):
    """ class to handle rendering of data """

    title = ''
    author = 'gord allott'

    callback = None
    locked = False

    frame = 0

    process = None
    iohup = None
    ioin = None

    proc_bufin = ""

    def __init__(self, title='', author=''):
        self.title = title
        self.author = author

    def findblend(self, filename='intro.blend'):
        """ finds a blend file on our special paths """
        if os.path.exists(resource.get_resource(filename)):
            return resource.get_resource(filename)
        else:
            raise Exception, '%s can not be found', filename

    def pad(self, value):
        """ simply pads "20" to "0020" and so on """
        newvalue = str(value)
        while len(newvalue) < 4:
            newvalue = "0" + newvalue

        return newvalue


    def _render(self, call_commands, callback, env=None):
        """ internal function designed to setup a program for running, runs
        callback when program is done
        """

        if not env:
            env = os.environ

        self.process = subprocess.Popen(call_commands, stdout=PIPE, stdin=PIPE, stderr=PIPE, env=env)

        self.iohup = gobject.io_add_watch(self.process.stdout,
                                          gobject.IO_HUP,
                                          self.callbackIO_HUP)

        # make sure that we start our io watch
        self.ioin = gobject.io_add_watch(self.process.stdout,
                                         gobject.IO_IN,
                                         self.callbackIO_IN)
        self.proc_bufin = ""

        self._callbackrender = callback


    def callbackIO_IN(self, source, condition):
        """
        checks the stdout of the process and handles it nicely.
        """
        #don't remove this, its needed for some reason... anyone know why?
        sys.stdout.write("f\b")

        buf = self.process.stdout.read(1)

        self.proc_bufin += buf
        #sys.stdout.write(buf)

        return True

    def callbackIO_HUP(self, source, condition):
        """
        simply called with the process has exited
        """
        gobject.source_remove(self.ioin)
        gobject.source_remove(self.iohup)

        self._callbackrender(self.proc_bufin)

        return False

    fadeout_start = 50.0 # we only start fading out at this 'percentage'

    def renderFadeout(self, callback, baseframe=75, newframe=76, fade=100.0):
        """ fades out based on baseframe, stores in newframe, with given fadeout
        percentage
        """

        fade = fade * (100.0 / self.fadeout_start)

        fade = cubicInterpolate(0.0, 0.0, 100.0, 100.0, fade / 100.0)

        fade = fade if fade < 100.0 else 100.0
        fade = fade if fade > 0.0 else 0.0

        if self.locked:
            raise Exception, 'function is locked'

        blur = str(round(20.0 / 100.0 * (100.0 - fade), 2))

        call_commands = ["convert"]
        call_commands += ["-modulate", str(round(fade, 4))]
        #call_commands += ["-gaussian-blur", "%sx%s" % (blur, blur)]
        call_commands += [os.path.join('/tmp', 'intro_' + self.pad(baseframe) + '.png')]
        call_commands += [os.path.join('/tmp', 'intro_' + self.pad(newframe) + '.png')]

        self.fadeout_outfilename = os.path.join('/tmp', 'intro_' + self.pad(newframe) + '.png')

        self.callback = callback

        self._render(call_commands, self._callbackRenderFadeout)

    def _callbackRenderFadeout(self, output):
        outfile =  self.fadeout_outfilename.lstrip().rstrip()

        if self.callback:
            self.callback('%s' % outfile)

    def renderFrame(self, callback, size=(128, 96), frame=75):
        """ renders one frame, once it is done it will call callback(filename)
        with filename being specified as where the frame was stored
        """
        if self.locked:
            raise Exception, 'function is locked'

        blendfile = self.findblend()
        script = resource.get_resource('buntu-text.py')
        call_commands = ["blender-bin",
                         "-b", blendfile,
                         "-t", "4",
                         "-F", "PNG",
                         "-x", "1",
                         "-o", r"/tmp/intro_",
                         "-P", script,
                         "-f", str(frame)]

        env = os.environ
        env['upeople_author'] = self.author
        env['upeople_title'] = self.title
        env['upeople_width'] = str(size[0])
        env['upeople_height'] = str(size[1])

        self.frame = frame

        self.callback = callback

        self._render(call_commands, self._callbackRenderFrame, env=env)

    def _callbackRenderFrame(self, output):
        regex_getframe = ".*\s(?P<filename>(\w|\s)+\.png)\sTime\:\s(?P<hours>\d+)\:(?P<minutes>\d+)\.(?P<seconds>\d+)"
        regex_getframe = r'Saved:.*?(?P<filename>.*\..*?) Time.*'
        reg = re.compile(regex_getframe)

        outfile = ""
        timetaken = ""

        for line in output.split("\n"):
            match = reg.search(line)

            if match:
                outfile = match.groupdict()['filename']
                #timetaken = match.groupdict()['hours'] + ":" + match.groupdict()['minutes'] + "." + match.groupdict()['seconds']

        outfile = outfile.lstrip().rstrip()

        if self.callback:
            self.callback('%s' % outfile)





class view(gui.view):
    """ gtk/glade object to provide the view """
    config = {
        'title': '',
        'description': '',
        'filepath': '',
    }

    paused = True

    updateTimer = None
    timerProgPulse = None

    seekHandle = None

    returnData = None

    renderer = None


    fullrender_frame = 0
    fullrender_frames = 75
    fullrender_size = (640, 480)

    fadeout_frame = 76
    fadeout_frames = 150


    animframes = []

    windowName = 'intro'

    encoder = None

    def __init__(self, config={}, wTree=None, callback=None):

        self.connectSignals = {
            'on_intro_cancelButton_clicked': self.cancelButton,
            'on_intro_okButton_clicked': self.applyButton,
            'on_intro_refresh_clicked': self.refreshButton,

            'on_intro_previewImage_size_allocate': self.imageResize,
        }
        gui.view.__init__(self, config=config, wTree=wTree, callback=callback)

        #lets set some default stuff
        self.wTree.get_widget('intro_title').set_text(self.config['title'])


    def startEncoding(self):
        self.wTree.get_widget('intro_progress').set_text("Encoding file")
        self.wTree.get_widget('intro_refresh').set_sensitive(False)
        self.wTree.get_widget('intro_author').set_sensitive(False)
        self.wTree.get_widget('intro_title').set_sensitive(False)
        self.wTree.get_widget('intro_okButton').set_sensitive(False)
        self.timerProgPulse = gobject.timeout_add(250, self.callbackProgPulse)


        self.encoder = Encoder(self.wTree.get_widget('intro_previewDrawable'))
        self.encoder.connect('ready', self.enc_readyCallback)
        self.encoder.connect('complete', self.enc_completeCallback)
        #print "start encoding here!"
        print self.encoder.sync()
        self.encoder.play()

    def enc_readyCallback(self, encoder):
        print "ready callback caught!"
        encoder.play()

    #def enc_checkCallback(self):
        #pos = self.encoder.get_position()
        #duration = self.encoder.get_duration()
        #if pos:
            #completed = float(pos)/float(duration)
        #else:
            #completed = 1.0
        #percent = 100*completed
        #print completed
        #self.wTree.get_widget('intro_progress').set_fraction(float(completed))
        #self.wTree.get_widget('intro_progress').set_text("%f%%" % percent)

        #return True

    def enc_completeCallback(self, encoder):
        gobject.source_remove(self.timerProgPulse)#self.updateTimer)
        self.wTree.get_widget('intro_progress').set_fraction(1.0)
        self.wTree.get_widget('intro_progress').set_text("")

        self.wTree.get_widget('intro_refresh').set_sensitive(True)
        self.wTree.get_widget('intro_author').set_sensitive(True)
        self.wTree.get_widget('intro_title').set_sensitive(True)
        self.wTree.get_widget('intro_okButton').set_sensitive(True)

        self.encoder = None



    # ----------- callbacks go here ------------

    def previewCallback(self, filename):
        """ callback for preview frame finishing """
        gobject.source_remove(self.timerProgPulse)

        self.wTree.get_widget('intro_refresh').set_sensitive(True)
        self.wTree.get_widget('intro_author').set_sensitive(True)
        self.wTree.get_widget('intro_title').set_sensitive(True)
        self.wTree.get_widget('intro_okButton').set_sensitive(True)

        #set our preview

        self.wTree.get_widget('intro_previewImage').set_from_file(filename)
        self.wTree.get_widget('intro_progress').set_fraction(0.0)


    def fullrenderCallback(self, filename):
        """ called back when a frame is done rendering whist doing a full
        render cycle
        """
        self.animframes += filename

        if self.timerProgPulse:
            gobject.source_remove(self.timerProgPulse)

        #set our preview
        self.wTree.get_widget('intro_previewImage').set_from_file(filename)

        #figure out our percentage done
        fraction = (1.0 / (self.fadeout_frames)) * self.fullrender_frame
        fraction_str = "Rendered frame %i of %i" % (self.fullrender_frame, (self.fadeout_frames))

        self.wTree.get_widget('intro_progress').set_fraction(fraction)
        self.wTree.get_widget('intro_progress').set_text(fraction_str)

        if self.fullrender_frame > self.fullrender_frames:
            self.fadeoutCallback('')
            return

        # update our done frames
        self.fullrender_frame += 1

        #get our title and author strings
        title = self.wTree.get_widget('intro_title').get_text()
        author = self.wTree.get_widget('intro_author').get_text()

        self.renderer = Render(title=title, author=author)

        self.renderer.renderFrame(  self.fullrenderCallback,
                                    size=self.fullrender_size,
                                    frame=self.fullrender_frame)

    def fadeoutCallback(self, filename):
        """ called back when a frame is done whist a fadeout is in effect
        """
        #set our preview
        if filename:
            self.wTree.get_widget('intro_previewImage').set_from_file(filename)

            #figure out our percentage done
            fraction = (1.0 / (self.fadeout_frames)) * self.fadeout_frame
            fraction_str = "Rendered frame %i of %i" % (self.fadeout_frame, (self.fadeout_frames))

            self.wTree.get_widget('intro_progress').set_fraction(fraction)
            self.wTree.get_widget('intro_progress').set_text(fraction_str)


        if self.fadeout_frame >= self.fadeout_frames:
            self.wTree.get_widget('intro_refresh').set_sensitive(True)
            self.wTree.get_widget('intro_author').set_sensitive(True)
            self.wTree.get_widget('intro_title').set_sensitive(True)
            self.wTree.get_widget('intro_okButton').set_sensitive(True)

            self.startEncoding()
            return



        fade = (100.0 / (self.fadeout_frames - self.fullrender_frames)) * (self.fadeout_frame - self.fullrender_frames)

        if self.fadeout_frame < 76:
            self.fadeout_frame = 76

        self.renderer = Render()
        self.renderer.renderFadeout( self.fadeoutCallback,
                                    baseframe = 75,
                                    newframe = self.fadeout_frame,
                                    fade = 100.0 - fade)

        self.fadeout_frame += 1



    def callbackProgPulse(self):
        """ callback that just dings the progress bar in its active state """
        self.wTree.get_widget('intro_progress').pulse()
        return True

    # ----------- Signals go here ------------


    def applyButton(self, widget):
        """ we assume the user wants to start rendering of the images and stuff
        here
        """
        self.wTree.get_widget('intro_refresh').set_sensitive(False)
        self.wTree.get_widget('intro_author').set_sensitive(False)
        self.wTree.get_widget('intro_title').set_sensitive(False)
        self.wTree.get_widget('intro_okButton').set_sensitive(False)
        title = self.wTree.get_widget('intro_title').set_sensitive(False)
        author = self.wTree.get_widget('intro_author').set_sensitive(False)

        #get our title and author strings
        title = self.wTree.get_widget('intro_title').get_text()
        author = self.wTree.get_widget('intro_author').get_text()

        #self.startEncoding()

        #self.renderer = Render(title=title, author=author)

        #self.renderer.renderFrame(self.fullrenderCallback, size=self.fullrender_size, frame=0)

        #self.timerProgPulse = gobject.timeout_add(250, self.callbackProgPulse)
        self.fadeoutCallback('')






    def refreshButton(self, widget):

        #get our 'allocation'
        allocation = self.wTree.get_widget('intro_previewImage').get_allocation()
        prewidth = allocation.width
        preheight = allocation.height

        #gives us our 4/3 aspect resolution
        width = int(prewidth)
        height = int((prewidth / 4) * 3)


        if height > preheight:
            deflate = float(preheight) / height
            height *= deflate
            width *= deflate

        height = int(height)
        width = int(width)


        #get our title and author strings
        title = self.wTree.get_widget('intro_title').get_text()
        author = self.wTree.get_widget('intro_author').get_text()

        self.renderer = Render(title=title, author=author)

        self.renderer.renderFrame(self.previewCallback, size=(width, height))

        self.timerProgPulse = gobject.timeout_add(100, self.callbackProgPulse)

        self.wTree.get_widget('intro_refresh').set_sensitive(False)
        self.wTree.get_widget('intro_author').set_sensitive(False)
        self.wTree.get_widget('intro_title').set_sensitive(False)
        self.wTree.get_widget('intro_okButton').set_sensitive(False)


    def imageResize(self, widget, allocation):
        """ called when the image size changes in gtk, image needs to be
        resized """
        prewidth = allocation.width
        preheight = allocation.height

        #gives us our 4/3 aspect resolution
        width = int(prewidth)
        height = int((prewidth / 4) * 3)


        if height > preheight:
            deflate = float(preheight) / height
            height *= deflate
            width *= deflate

        height = int(height)
        width = int(width)

        #we now need to resize our image (if any)
        try:
            pixbuf = self.wTree.get_widget('intro_previewImage').get_pixbuf()
        except:
            return

        if not pixbuf:
            return

        #!!FIXME - for some reason the following .set_from_puxbuf() cod forces
        # the entire program to hang eating up all the cpu in the world. thus
        # i am going to disable this and fix it at some less busy point

        #pixbuf = pixbuf.scale_simple(width, height, gtk.gdk.INTERP_BILINEAR)
        #self.wTree.get_widget('intro_previewImage').set_from_pixbuf(pixbuf)





