#! /usr/bin/env python
# -*- coding: utf-8 -*-
#    This file is part of ubuntu-uploader.
#
#    ubuntu-uploader is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    ubuntu-uploader is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with ubuntu-uploader.  If not, see <http://www.gnu.org/licenses/>.


import sys, os
import pygtk
pygtk.require('2.0')
import gtk, gtk.glade, gobject
import resource, iconHandler

import guiScreencast
import guiUploadConfig
import guiUploader
import guiPreview
import guiIntroConfig
import guiMediaEditing

from resource import extVideo, extAudio, mb_it
import datetime
from math import *
import pygst
pygst.require("0.10")
import gst

import httplibnb, gsocket

from os import path as ospath

def FileChooser():
    """ opens a file chooser dialog and returns the file choosen """
    dialog = gtk.FileChooserDialog("Open..",
                                    None,
                                    gtk.FILE_CHOOSER_ACTION_OPEN,
                                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                    gtk.STOCK_OPEN, gtk.RESPONSE_OK))
    dialog.set_default_response(gtk.RESPONSE_OK)

    filter = gtk.FileFilter()
    filter.set_name("Media")
    for ext in extVideo:
        filter.add_mime_type('video/' + ext)
        filter.add_pattern('*.' + ext)

    for ext in extAudio:
        filter.add_mime_type('audio/' + ext)
        filter.add_pattern('*.' + ext)

    dialog.add_filter(filter)



    response = dialog.run()
    filename = ""

    if response == gtk.RESPONSE_OK:
        filename =  dialog.get_filename()
    elif response == gtk.RESPONSE_CANCEL:
        filename = None
    dialog.destroy()

    return filename

class IconView(object):
    """ gtk icon view handling """

    widget = None
    model = None

    def __init__(self, widget):
        """ initalises the object, widget is the iconview widget that we will
        be handling
        """
        # our model has its displayed text as the first item, an image as the
        # second and the third is not displayed, should be a unique identifier
        # such as a UUID

        self.widget = widget
        self.model = gtk.ListStore(str, gtk.gdk.Pixbuf, str)
        self.widget.set_model(self.model)

    def addItem(self, title, UUID, icon=(None, None)):

        # first check that the uuid doesn't already exist in the liststore
        for index in range(len(self.model)):
            if self.model[index][2] == UUID:
                return

        if icon[0] == "stock":
            #do a stock icon
            pixbuf = self.widget.render_icon(icon[1], gtk.ICON_SIZE_LARGE_TOOLBAR,)

        elif icon[0] == "path":
            #do an icon defined by a path
            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(
                        icon[1], 128, 96)

        elif icon[0] == "pixbuf":
            # just use the supplied pixbuf
            pixbuf = icon[1]

        else:
            #default icon
            pixbuf = self.widget.render_icon(   gtk.STOCK_MISSING_IMAGE,
                                                gtk.ICON_SIZE_LARGE_TOOLBAR,)

        self.model.append([title, pixbuf, UUID])

    def getItem(self, index):
        """ returns the UUID of the item at the given index in the model """
        return self.model[index][2]

    def removeItem(self, uuid):
        """ removes the item with the given uuid from the model """
        for index in range(len(self.model)):
            if self.model[index][2] == uuid:
                self.model.remove(self.model.get_iter(index))
                break




class InfoView(object):



    Info = {
        'title'     : 'nothing selected',
        'icon'      : None,
        'length'    : 0,
        'filesize'  : 0,
        'filepath'  : None,
    }

    def __init__(self, wTree):
        """
        initalises the object, widget is the wTree that we need to grab our
        widgets
        """
        # our model has its displayed text as the first item, an image as the
        # second and the third is not displayed, should be a unique identifier
        # such as a UUID

        self.wTree = wTree


    def updateInfo(self):
        """ updates the info in the info widgets """

        length = [  int(floor(self.Info['length'] / 60 / 60)),          # hours
                    int(floor(self.Info['length'] / 60) % 60),          # minutes
                    int(floor(self.Info['length']) % 60),               # seconds
                    int(floor(
                        (self.Info['length'] - floor(self.Info['length'])) * 100
                    ))    # miliseconds
        ]

        def pad(num):
            if len(str(num)) == 1:
                return "0" + str(num)
            return str(num)

        length[0] = str(length[0])
        length[1] = pad(length[1])
        length[2] = pad(length[2])
        length[3] = str(length[3])

        len_str = "%s:%s:%s.%s" % tuple(length)

        self.wTree.get_widget('info_length').set_text(len_str)

        self.wTree.get_widget('info_filesize').set_text(mb_it(self.Info['filesize']))
        self.wTree.get_widget('info_title').set_text(self.Info['title'])
        self.wTree.get_widget('info_filepath').set_text(self.Info['filepath'])


    def gatherInfo(self, filepath, title=None):
        """
        gathers info from the filepath given, also will use a title if one is
        given, if its not the file name will be used as a title
        """
        #check the file exists
        if not ospath.exists(filepath):
            #should never get here
            raise Exception

        #sets our title
        title = title if title else ospath.split(filepath)[1]

        #find the length of our media
        #FIXME!! - actually make this happen
        length = 165

        #find the filesize of our file
        filesize = ospath.getsize(filepath)

        self.Info['title'] = title
        self.Info['length'] = length
        self.Info['filesize'] = filesize
        self.Info['filepath'] = filepath





class view(object):
    """ gtk/glade object to provide the view """

    library = None
    iconView = None
    screencastView = None
    uploadView = None
    previewView = None
    mediaEditingView = None

    def __init__(self, wTree=None):
        """
        inits the view, if you pass it a wTree it will use that instead of
        creating a new wTree instance
        """

        #replace this with nicer lookups
        self.gladefile = resource.get_resource("glade/app.glade")

        self.wTree = wTree if wTree else gtk.glade.XML(self.gladefile)
        self.window = self.wTree.get_widget("uu-library")
        if self.window:
            self.window.connect('destroy', self.destroy)

        #create our iconview object
        self.iconView = IconView(self.wTree.get_widget('iconview_media'))

        #create our infoview object
        self.infoView = InfoView(self.wTree)

        #connect up a few signals

        dic = {
            "on_toolbutton_addMedia_clicked" : self.addMedia_clicked,
            "on_toolbutton_removeMedia_clicked": self.removeMedia_clicked,
            "on_toolbutton_recordScreencast_clicked": self.joinMedia, #self.recordScreencast_clicked,

            "on_iconview_media_selection_changed": self.media_selection_changed,

            "on_uu-library_check_resize": self.changesizepos,

            "on_toolbutton_uploadVideo_clicked": self.uploadVideo,

            "on_imagemenuitem10_activate": self.showAbout, #About menu item

            "on_toolbutton_previewMedia_clicked": self.previewMedia,

            "on_toolbutton_attachIntro_clicked": self.attachIntro,

            "on_uu-library_destroy" : gtk.main_quit
        }

        #disable a few buttons
        self.wTree.get_widget("toolbutton_removeMedia").set_sensitive(False)
        self.wTree.get_widget("toolbutton_uploadVideo").set_sensitive(False)

        self.wTree.signal_autoconnect(dic)


        width, height, x, y = self.getwidthheight()
        if x > 0 and y > 0:
            self.window.move(x, y)

        self.window.resize(width, height)


        #show our window
        self.window.show()

    def getwidthheight(self):
        """
        Tries to get the last used width and height from a config file
        if it doesn't exist (or their are problems parsing it) then we use
        a default
        """
        user_dir = os.path.expanduser('~/.ubuntu-uploader/')
        if not os.path.exists(user_dir):
            try:
                os.mkdir(user_dir)
            except:
                print "Error: could not create user directory in " + user_dir +\
                "please check your permissions"

        if os.path.exists(os.path.join(user_dir, 'lastwindow')):
            tfile = open(os.path.join(user_dir, 'lastwindow'), 'r')
            line = tfile.readline()
            line = line.lstrip().rstrip()

            width, height, x, y = line.split(" ")

            width = int(width)
            height = int(height)
            x = int(x)
            y = int(y)

            return (width, height, x, y)
        else:
            return (640, 480, -1, -1)

    def setwidthheight(self):
        """
        tries to set the current width and height to a config file
        """
        user_dir = os.path.expanduser('~/.ubuntu-uploader/')
        tfile = None
        if not os.path.exists(user_dir):
            try:
                os.mkdir(user_dir)
                tfile = open(os.path.join(user_dir, 'lastwindow'), 'w')
            except:
                print "Error: could not create user directory in " + user_dir +\
                "please check your permissions"
                raise Exception

        tfile = open(os.path.join(user_dir, 'lastwindow'), 'w')

        width, height = self.window.get_size()
        x, y = self.window.get_position()

        tfile.write(str(width) + ' ' + str(height) + ' ' + str(x) + ' ' + str(y))
        tfile.close()


    def attachLibrary(self, libraryObject):
        """ attaches the given library object to the current object """
        self.library = libraryObject

        self.buildIconView()


    def buildIconView(self):
        """ takes the library object and does a first build on it,
        basically should only be called when we are first loading
        """
        #add our current items to the Iconview
        for key, item in self.library.iteritems():
            title = ""

            #check to see if our title is empty, else use filename
            if item['title'].rstrip():
                title = item['title'].rstrip()
            else:
                title = os.path.split(item['filepath'])[1]

            #get the icon for the filepath
            icon = iconHandler.getIcon(item['filepath'])

            self.iconView.addItem(title, item['UUID'], icon=icon)

    def callbackScreencast(self, data=None):
        """ just a simple callback for our screencast thingy """
        self.window.show()
        self.screencastView = None

        #add our new media if we have any
        if data:
            if data.has_key('filename'):
                uuid = self.library.addMedia(data['filename'])
            else:
                return


            if data.has_key('title') and data['title']:
                title = data['title']
            else:
                title = os.path.split(self.library[uuid]['filepath'])[1]

            icon = iconHandler.getIcon(self.library[uuid]['filepath'])

            self.library[uuid]['title'] = title
            self.library[uuid]['description'] = data['description']

            self.iconView.addItem(title, uuid, icon=icon)

    def callbackUpload(self, data=None):
        """ callback for uploader """
        self.window.show()
        self.uploadView = None

    def callbackPreview(self, data=None):
        """ callback for media preview """
        self.window.show()
        self.previewView = None

    def callbackIntro(self, data=None):
        """ callback for media preview """
        self.window.show()
        self.introView = None


    #------------------------- Signals go here --------------------------------

    def destroy(self, widget):
        #self.setwidthheight()
        self.library.saveXML()
        gtk.main_quit()

    def changesizepos(self, widget):
        self.setwidthheight()

    def showAbout(self, widget):
        """ shows the about window """
        self.wTree.get_widget("aboutdialog").show()

    def addMedia_clicked(self, widget):

        #open a file chooser dialog
        chosen_filename = FileChooser()

        #add our new filename to our library
        #!!FIXME - add checks here, we don't want to add *anything* to our library

        uuid = self.library.addMedia(chosen_filename)

        #use the uuid we just got to add that media item to the iconview
        title = os.path.split(self.library[uuid]['filepath'])[1]

        #get our icon
        icon = iconHandler.getIcon(self.library[uuid]['filepath'])

        self.iconView.addItem(title, uuid, icon=icon)



    def removeMedia_clicked(self, widget):
        selectedIcons = self.wTree.get_widget('iconview_media').get_selected_items()
        if not selectedIcons:
            #we should never get here
            self.wTree.get_widget("toolbutton_removeMedia").set_sensitive(False)
            return

        #get a list of the uuids returned
        uuids = []
        for item in selectedIcons:
            uuids.append(self.iconView.getItem(item))

        for uuid in uuids:
            #get the uuid of the item

            if not uuid:
                pass

            self.library.pop(uuid)
            self.iconView.removeItem(uuid)


    def media_selection_changed(self, widget):
        """ called when the selection of media files changes """
        selected = widget.get_selected_items()
        if selected:
            self.wTree.get_widget("toolbutton_removeMedia").set_sensitive(True)

            if len(selected) > 1:
                self.wTree.get_widget("toolbutton_uploadVideo").set_sensitive(False)
                self.wTree.get_widget("toolbutton_previewMedia").set_sensitive(False)
                self.wTree.get_widget("toolbutton_attachIntro").set_sensitive(False)
            else:
                self.wTree.get_widget("toolbutton_uploadVideo").set_sensitive(True)
                self.wTree.get_widget("toolbutton_previewMedia").set_sensitive(True)
                self.wTree.get_widget("toolbutton_attachIntro").set_sensitive(True)


        else:
            self.wTree.get_widget("toolbutton_removeMedia").set_sensitive(False)
            self.wTree.get_widget("toolbutton_uploadVideo").set_sensitive(False)
            self.wTree.get_widget("toolbutton_previewMedia").set_sensitive(False)
            self.wTree.get_widget("toolbutton_attachIntro").set_sensitive(False)
            return

        # change the info widgets
        # for the moment only provide info for the first widget

        uuid = self.iconView.getItem(selected[0])
        filepath = self.library[uuid]['filepath']
        title = self.library[uuid]['title']

        self.infoView.gatherInfo(filepath, title=title)
        self.infoView.updateInfo()


    def recordScreencast_clicked(self, widget):
        """ triggered when we need to do screencast stuff"""
        #we need to make this better!
        self.window.hide()
        self.screencastView = guiScreencast.view(wTree=self.wTree,
                                            callback=self.callbackScreencast)

    def uploadVideo(self, widget):
        """ triggered when we need to upload the currently selected media"""
        selectedIcons = self.wTree.get_widget('iconview_media').get_selected_items()
        uuid = self.iconView.getItem(selectedIcons[0])

        config = {
            'title': self.library[uuid]['title'],
            'description': self.library[uuid]['description'],
            'filepath': self.library[uuid]['filepath'],
        }


        self.window.hide()
        self.uploadView = guiUploader.view(config=config, wTree=self.wTree, callback=self.callbackUpload)

    def previewMedia(self, widget):
        """ triggered when the preview media button is called to preview the
        currently selected media
        """
        selectedIcons = self.wTree.get_widget('iconview_media').get_selected_items()
        uuid = self.iconView.getItem(selectedIcons[0])

        config = {
            'title': self.library[uuid]['title'],
            'description': self.library[uuid]['description'],
            'filepath': self.library[uuid]['filepath'],
        }
        self.window.hide()
        self.previewView = guiPreview.view(config=config, wTree=self.wTree, callback=self.callbackPreview)


    def attachIntro(self, widget):
        """ triggered when the attach window button is called """

        selectedIcons = self.wTree.get_widget('iconview_media').get_selected_items()
        uuid = self.iconView.getItem(selectedIcons[0])

        config = {
            'title': self.library[uuid]['title'],
            'description': self.library[uuid]['description'],
            'filepath': self.library[uuid]['filepath'],
        }

        self.window.hide()
        self.introView = guiIntroConfig.view(config=config, wTree=self.wTree, callback=self.callbackIntro)


    def callbackEditing(self, data=None):
        """ callback for media preview """
        self.window.show()
        self.mediaEditingView = None
        print "DONE!!!! :D.. or not"

    def joinMedia(self, widget):
        """ triggered when the join media button is called """

        config = {
            'title': "testfile",
            'description': "this is a test file, isn't it spiffy",
            'fps': 25,
            'audioQuality': 75.0,
            'videoQuality': 100.0,
            'saveLocation': "/home/gord/Videos/testfile.ogg",
            'inputFiles': [
                #'/tmp/intro.ogg',
                '/home/gord/Videos/Testvideo2.ogg',
                '/home/gord/Videos/TestVideo1.ogg',
            ]
        }

        self.mediaEditingView = guiMediaEditing.view(config=config, wTree=self.wTree, callback=self.callbackEditing)




