#!/usr/bin/env python
# -*- coding: utf-8 -*-
#       guiUploader.py
#
#       Copyright 2008 Gordon Allott <gordallott@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#
import sys, os
import pygtk
pygtk.require('2.0')
import gtk, gtk.glade, gobject
import resource, iconHandler, guiUploadConfig
from resource import extVideo, extAudio, mb_it
import datetime
from math import *
import pygst
pygst.require("0.10")
import gst
from gst.extend.discoverer import Discoverer

import re
from os import path as ospath
from resource import slugify
from time import sleep
import gui
import gstHelpers
from random import random

class AudioDec(gst.Bin):
    """Decodes audio file, outputs at specified caps"""

    def __init__(self, location, caps=None):
        gst.Bin.__init__(self)

        if not caps:
            caps = gst.caps_from_string('audio/x-raw-float, rate=44100, channels=2, endianness=1234, width=32')


        # Create elements
        src = gst.element_factory_make('filesrc')
        dec = gst.element_factory_make('decodebin')
        conv = gst.element_factory_make('audioconvert')
        rsmpl = gst.element_factory_make('audioresample')
        ident = gst.element_factory_make('identity')

        # Set 'location' property on filesrc
        src.set_property('location', location)

        # Connect handler for 'new-decoded-pad' signal
        dec.connect('new-decoded-pad', self.__on_new_decoded_pad)

        # Add elements to bin
        self.add(src, dec, conv, rsmpl, ident)

        # Link *some* elements
        # This is completed in self.__on_new_decoded_pad()
        src.link(dec)
        conv.link(rsmpl)
        rsmpl.link(ident, caps)

        # Reference used in self.__on_new_decoded_pad()
        self.__apad = conv.get_pad('sink')

        # Add ghost pad
        self.add_pad(gst.GhostPad('src', ident.get_pad('src')))


    def __on_new_decoded_pad(self, element, pad, last):
        caps = pad.get_caps()
        name = caps[0].get_name()
        #print '\n__on_new_decoded_pad:', name
        if 'audio' in name:
            if not self.__apad.is_linked(): # Only link once
                pad.link(self.__apad)


class VideoDefault(gst.Bin):
    """ we use this as our default video source, if nothing else is playing,
    this will
    """

    def __init__(self):

        gst.Bin.__init__(self)

        self._source = gst.element_factory_make('videotestsrc')
        self._source.set_property('pattern', 'snow')
        self._convert = gst.element_factory_make('ffmpegcolorspace')
        self._identity = gst.element_factory_make('identity')

        self.add(self._source, self._convert, self._identity)
        self._source.link(self._convert)
        self._convert.link(self._identity)

        self.add_pad(gst.GhostPad('src', self._identity.get_pad('src')))

class VideoDec(gst.Bin):
    """Decodes audio file, outputs at specified caps"""


    salt = 1.23456

    def __init__(self, location, caps=None, salt=0.0, callback=None):
        gst.Bin.__init__(self)

        self.salt = salt
        self.callback = callback

        if not caps:
            #print "redoing caps.."
            caps = gst.caps_from_string('video/x-raw-yuv, framerate=(fraction)25/1')
            #print caps

        #print type(caps), caps

        # Create elements
        self._source = gst.element_factory_make('filesrc')
        self._decode = gst.element_factory_make('decodebin')
        self._convert = gst.element_factory_make('ffmpegcolorspace')
        #resize = gst.element_factory_make('videoscale')
        #rate = gst.element_factory_make('videorate')
        self._identity = gst.element_factory_make('identity')

        # Set 'location' property on filesrc
        self._source.set_property('location', location)

        # Connect handler for 'new-decoded-pad' signal
        self._decode.connect('new-decoded-pad', self.__on_new_decoded_pad)

        # Add elements to bin
        self.add(self._source, self._decode, self._convert, self._identity)

        # Link *some* elements
        # This is completed in self.__on_new_decoded_pad()
        self._source.link(self._decode)
        self._convert.link(self._identity)
        #resize.link(rate, caps)
        #rate.link(identity, caps)

        # Reference used in self.__on_new_decoded_pad()
        self.__apad = self._convert.get_pad('sink')

        # Add ghost pad
        self.add_pad(gst.GhostPad('src', self._identity.get_pad('src')))


    def __on_new_decoded_pad(self, element, pad, last):
        caps = pad.get_caps()
        name = caps[0].get_name()
        #print '\n__on_new_decoded_pad:', name
        if 'video' in name:
            if not self.__apad.is_linked(): # Only link once
                pad.link(self.__apad)
                gobject.idle_add(self.callback, self, self.salt)

    def set_state(self, state):
        """ replacement for the missing set_state included in normal elements """
        self._source.set_state(state)
        self._decode.set_state(state)
        self._convert.set_state(state)
        self._identity.set_state(state)

gobject.type_register(VideoDec)

class Editor(gobject.GObject):
    __gsignals__ = {
            'ready' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
            'complete' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
            'error' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
            'paused' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
            'playing' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
            'eos' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [])
        }



    hookedup = False

    def __init__(self, output='/tmp/out.ogg'):
        gobject.GObject.__init__(self)
        self._pipeline = gst.Pipeline("mypipeline")




        #self._videoencoder = gstHelpers.VideoEncoder(videorate=False)
        #self._audioencoder = gstHelpers.AudioEncoder()

        self._mux = gst.element_factory_make('oggmux', 'mux')
        #self._pipeline.add(self._mux)

        self._progreport = gst.element_factory_make('progressreport')

        self._filesink = gst.element_factory_make('filesink', 'filesink')
        self._filesink.set_property('location', output)

        #self._pipeline.add(#self._videoencoder,
                           ##self._audioencoder,
                           ##self._mux,
                           ##self._progreport,
                           #self._filesink)


        # creating a gnlcomposition
        self._comp = gst.element_factory_make("gnlcomposition", "mycomposition")
        self._comp.connect("pad-added", self.OnPad)
        self._comp.connect("no-more-pads", self.no_more_pads)

        #self._videorate = gst.element_factory_make("videorate", "videoEncoder-videorate")
        #self._videocaps = gst.caps_from_string('video/x-raw-yuv,framerate=25/1')

        self._videoencoder = gstHelpers.VideoEncoder(framerate=25)
        #self._videoencoder.set_property("caps", self._videocaps)
        #self._videoencoder.set_state(gst.STATE_PAUSED)

        self._pipeline.add(self._comp, self._mux, self._filesink, self._videoencoder)
        self._mux.link(self._filesink)
        #self._videorate.link(self._videoencoder)
        self._videoencoder.link(self._mux)


        #gst.element_link_many(self._videoencoder,
                              #self._mux)

        #gst.element_link_many(self._audioencoder,
                              #self._mux)

        #gst.element_link_many(self._mux,
                              ##self._progreport,
                              #self._filesink)

        # Connect up callbacks
        bus = self._pipeline.get_bus()
        bus.add_signal_watch()
        bus.connect("message", self.on_message)
        bus.connect("message::state-changed", self._on_state_changed)

        self._sources = []

    audioConnected = False
    videoConnected = False


    def defaultSources(self):
        """ called to setup the default sources, needs to have all the files
        added to our gnonlin thingy first though
        """
        #setup our default video source
        #self._defaultVideosource = VideoDefault()

        ## create a gnlsource
        #media = gst.element_factory_make("gnlsource", 'defaultVideoSource')
        #media.add(self._defaultVideosource)

        ## set the gnlfilesource properties
        ##media.set_property("location", filepath)
        #media.set_property("start", 0)
        #media.set_property("duration", self.addmedia_currentpos)
        #media.set_property("priority", 9064)
        ##media.set_property("media-start", 0)
        ##media.set_property("media-duration", self.addmedia_currentpos)
        #media.set_state(gst.STATE_PAUSED)
        #self._comp.add(media)

        #self.sync(media)

        pass

    addmedia_currentpos = 0      # we use this to track the 'start' of each media as its added

    def addMedia(self, filepath, start, end):
        start = int(start * gst.SECOND)
        end = int(end * gst.SECOND)
        salt = random() #salt is used to identify this media again later

        id = "%s[%f]" % (filepath, salt)

        duration = end - start

        print "addMedia", start, end, duration

        #make our decoder for video first
        vid_decoder = VideoDec(filepath, salt=salt, callback=self.onmediaReady)


        # create a gnlfilesource
        media = gst.element_factory_make("gnlsource", id)
        media.add(vid_decoder)

        # set the gnlfilesource properties
        #media.set_property("location", filepath)
        media.set_property('start', self.addmedia_currentpos)
        media.set_property('duration', duration)
        media.set_property('media-start', start)
        media.set_property('media-duration', duration)
        #media.set_property('priority', 1)
        media.set_state(gst.STATE_PAUSED)


        self._sources.append((media, salt, False))
        self._comp.add(media)

        self.addmedia_currentpos += duration

    def areweready(self):
        """ simple function that can be called to check various things to make
        sure we are ready to start
        """
        print "|----| testing readyness"
        for media, salt, ready in self._sources:
            if not ready:
                print "salt %f is not ready" % salt
                print self._sources
                return False

        if not self.hookedup:
            print "self.hookedup is not true"
            #self.pause()
            return False

        print "|----| We are ready here apprently\n"
        #if we get here we are ready
        self._comp.set_property("duration", self.addmedia_currentpos)
        self.defaultSources()
        self.emit('ready')
        return True


    def sync(self, element=None):
        """
        Wait for the pipeline to change state
        We give it a 3 second timeout: returns True if the state change was
        successful in this time, False otherwise
        """
        if not element:
            element = self._pipeline

        state = element.get_state(timeout=3*gst.SECOND)[0]
        if state == gst.STATE_CHANGE_SUCCESS:
            return True
        else:
            return False

    def ready(self):
        """ called to emit the ready signal """
        self.sync()
        self.emit('ready')
        return False

    def play(self):
        print "<---- play requested"
        #self._comp.set_state(gst.STATE_PLAYING)
        self._pipeline.set_state(gst.STATE_PLAYING)

    def pause(self):
        print "<---- pause requested"
        self._pipeline.set_state(gst.STATE_PAUSED)

    def stop(self):
        print "<---- stop requested"
        self.bus.remove_signal_watch()
        self._pipeline.set_state(gst.STATE_NULL)

    def get_position(self):
        try:
            return self._pipeline.query_position(gst.FORMAT_TIME)[0]
        except gst.QueryError:
            return None

    def get_duration(self):
        try:
            return self._pipeline.query_duration(gst.FORMAT_TIME)[0]
        except gst.QueryError:
            return None

    # -------------------------------------------------------------------

    def onmediaReady(self, element, mediaSalt):
        """ handles the ready signals from our media """
        print "media %f is ready!" % mediaSalt, element
        for index in range(len(self._sources)):
            media, salt, ready = self._sources[index]
            if mediaSalt == salt:
                print "source test; ", self._sources[index]
                self._sources[index] = (media, salt, True)
                print "source test; ", self._sources[index]
                self.areweready()
                return

        raise Exception, 'onmediaReady called but salt was not found, salt was:', salt


    def OnPad(self, comp, pad):
        #print "pad added!"
        caps = pad.get_caps()
        pad_type = caps.to_string()[0:5]
        #print pad, caps.to_string()

        print pad_type

        if pad_type == "video":
            #self._videoencoder = gstHelpers.VideoEncoder(videorate=False)
            #self._pipeline.add(self._videoencoder)
            #self._videoencoder.set_state(gst.STATE_PAUSED)
            #self._videoencoder.set_property("caps", self._videocaps)

            pad.link(self._videoencoder.get_compatible_pad(pad, pad.get_caps()))#get_pad("sink"))
            #pad.link(self._videorate.get_pad('sink'))
            #self._videorate.link(self._videoencoder)
            #self._pipeline.add(self._mux, self._filesink)
            #self._mux.link(self._filesink)
            #self._videoencoder.link(self._mux)
            #self.ready()
            pass

        elif pad_type == "audio":
            self._audioencoder = gstHelpers.AudioEncoder()
            self._audioencoder.set_state(gst.STATE_PAUSED)
            pad.link(self._audioencoder.get_pad("sink"))
            self._audioencoder.link(self._mux)
            #self.ready()

        else:
            print "Unknown pad type detected, %s" %pad_type

        #convpad = self.compconvert.get_compatible_pad(pad, pad.get_caps())
        #pad.link(convpad)

    def no_more_pads(self, element):
        print "done adding pads"
        print "self.hookedup is true :)"
        self.hookedup = True
        self.areweready()
        #self.emit('ready')

    def _on_state_changed(self, bus, message):
        state = message.parse_state_changed()

        if  (message.src == self._pipeline):

            if state[1] == gst.STATE_PAUSED:
                print "----> Stream is Paused "
                self.emit('paused')
            elif state[1] == gst.STATE_PLAYING:
                print "----> Stream is Playing "
                self.emit('playing')

        else:
            first = "-" #✗□◼▶
            second = "\t" + str(message.src)
            if state[1] == gst.STATE_PAUSED:
                first = u'◼'.encode('utf-8')

            if state[1] == gst.STATE_PLAYING:
                first = u'▶'.encode('utf-8')

            if state[1] == gst.STATE_READY:
                first = u'□'.encode('utf-8')


            if state[1] == gst.STATE_NULL:
                first = u'✗'.encode('utf-8')

            print first + second


        #if  (message.src == self._pipeline):
            #state = message.parse_state_changed()

            ##if state[1] == gst.STATE_PAUSED:
                ##self.emit('paused')
            ##elif state[1] == gst.STATE_PLAYING:
                ##self.emit('playing')

    def on_message(self, bus, message):

        type = message.type
        if type == gst.MESSAGE_EOS:
            print "eos"
            self._pipeline.set_state(gst.STATE_NULL)
            self.sync()
            self.emit('complete')

        elif type == gst.MESSAGE_ERROR:
            print "error"
            self._pipeline.set_state(gst.STATE_NULL)
            err, debug = message.parse_error()
            print "Error: %s" % err, debug
            self.emit('error')




class view(gui.view):
    """ gtk/glade object to provide the view """

    config = {
        'title': "",
        'description': "",
        'fps': 15,
        'audioQuality': 75.0,
        'videoQuality': 100.0,
        'saveLocation': "~/Videos/foo.ogg",
        'inputFiles': []
    }

    inputfiles = []
    currentfile = 0

    windowName = 'mediaAction'

    editor = None

    timerProgPulse = None

    def __init__(self, config=None, wTree=None, callback=None):

        if not config:
            config = self.config

        self.connectSignals = {
            "on_mAction_ok_clicked" : self.okayButton,
            "on_mAction_cancel_clicked" : self.cancelButton,
        }

        gui.view.__init__(self, config=config, wTree=wTree, callback=callback)

        #print "given input files: ", self.config['inputFiles']

        if not self.config['inputFiles']:
            self.destroy()

        self.editor = Editor(self.config['saveLocation'])
        self.editor.connect('ready', self.on_ready)

        gobject.idle_add(self.discover)

        # start a nice pulse for the progress bar
        self.timerProgPulse = gobject.timeout_add(250, self.callbackProgPulse)


    def callbackProgPulse(self):
        """ callback that just dings the progress bar in its active state """
        self.wTree.get_widget('mAction_progress').pulse()
        return True



    def discover(self):
        infile = self.config['inputFiles'][self.currentfile]
        #print "trying to descover: %s" , infile
        discoverer = Discoverer(infile)
        discoverer.connect('discovered', self.on_discovered, infile)
        discoverer.discover()
        return False # Don't repeat idle call


    def on_discovered(self, discoverer, ismedia, infile):
        #print '\non_discovered:', infile
        discoverer.print_info()
        #if discoverer.is_audio:
            #dec = AudioDec(infile, self.caps)
            #src = gst.element_factory_make('gnlsource')
            #src.add(dec)
            #src.set_property('media-start', 0L)
            #src.set_property('media-duration', discoverer.audiolength)
            #src.set_property('start', self.start)
            #src.set_property('duration', discoverer.audiolength)
            #self.comp.add(src)
            #self.start += discoverer.audiolength
        #print "length: ", float(discoverer.audiolength) / gst.SECOND
        self.editor.addMedia(infile, 0, float(discoverer.videolength) / gst.SECOND)

        self.currentfile += 1
        if self.currentfile < len(self.config['inputFiles']):
            gobject.idle_add(self.discover)
            print "continue..."
        else:
            #self.pipeline.set_state(gst.STATE_PLAYING)
            #print "start!"
            self.editor.pause()
            self.editor.sync()
            pass

    def on_ready(self, editor):
        print "On_ready"
        #editor.sync()
        editor.play()

        #print "Playing...", editor.sync()
        return True
