#!/usr/bin/env python
# -*- coding: utf-8 -*-
#       guiUploader.py
#
#       Copyright 2008 Gordon Allott <gordallott@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#
import sys, os
import pygtk
pygtk.require('2.0')
import gtk, gtk.glade, gobject
import resource, iconHandler, guiUploadConfig
from resource import extVideo, extAudio, mb_it
import datetime
from math import *
import pygst
pygst.require("0.10")
import gst

import re
from os import path as ospath
from resource import slugify
from time import sleep
import gui

class view(gui.view):
    """ gtk/glade object to provide the view """

    config = {
        'title': '',
        'description': '',
        'filepath': '',
    }

    paused = True

    timerUpdate = None
    seekHandle = None

    windowName = 'preview'

    def __init__(self, config=None, wTree=None, callback=None):



        self.connectSignals = {
            "on_preview_playButton_clicked" : self.togglePlay,
            'on_preview_closeButton_clicked': self.cancelButton,
        }
        gui.view.__init__(self, wTree=wTree, callback=callback, config=config)

        self.seekHandle = self.wTree.get_widget('preview_seekScale'
                            ).connect('value-changed', self.seek)


        # set our title n stuff
        self.wTree.get_widget("preview_Titlelabel").set_label("<b>%s</b>" % self.config['title'])
        self.wTree.get_widget("preview_timeLabel").set_text("00:00 / 00:00")



        #handle our gst stuff
        self.player = gst.element_factory_make("playbin", "player")
        bus = self.player.get_bus()
        bus.add_signal_watch()
        bus.enable_sync_message_emission()
        bus.connect("message", self.on_message)
        bus.connect("sync-message::element", self.on_sync_message)

        filepath = self.config['filepath']

        if os.path.isfile(filepath):
            self.player.set_property('uri', "file://" + filepath)

        self.videoWidget = self.wTree.get_widget("preview_contentArea")

        self.time_format = gst.Format(gst.FORMAT_TIME)

        self.player.set_state(gst.STATE_PLAYING)
        self.player.set_state(gst.STATE_PAUSED)

        self.callbackUpdate()



    def convertTime(self, time_int):
        time_int = time_int / 1000000000
        time_str = ""
        if time_int >= 3600:
            _hours = time_int / 3600
            time_int = time_int - (_hours * 3600)
            time_str = str(_hours) + ":"

        if time_int >= 600:
            _mins = time_int / 60
            time_int = time_int - (_mins * 60)
            time_str = time_str + str(_mins) + ":"

        elif time_int >= 60:
            _mins = time_int / 60
            time_int = time_int - (_mins * 60)
            time_str = time_str + "0" + str(_mins) + ":"

        else:
            time_str = time_str + "00:"

        if time_int > 9:
            time_str = time_str + str(time_int)

        else:
            time_str = time_str + "0" + str(time_int)

        return time_str




    def callbackUpdate(self):
        """ should use a timer to update the screen """
        try:
            dur = self.player.query_duration(self.time_format, None)[0]
            strdur = self.convertTime(dur)
        except:
            dur = 1.0
            strdur = '00:00'

        try:
            pos = self.player.query_position(self.time_format, None)[0]
            strpos = self.convertTime(pos)
        except:
            pos = 0.000001
            strpos = '00:00'

        timeInfo = "%s / %s" % (strpos, strdur)
        progress = (float(pos) / dur) * 100.0

        self.wTree.get_widget("preview_seekScale").handler_block(self.seekHandle)

        self.wTree.get_widget("preview_timeLabel").set_text(timeInfo)
        self.wTree.get_widget("preview_seekScale").set_value(progress)

        self.wTree.get_widget("preview_seekScale").handler_unblock(self.seekHandle)

        return True


    def pause(self):
        """ pauses the current gst player """
        self.wTree.get_widget('preview_playButton').set_label(gtk.STOCK_MEDIA_PLAY)
        self.player.set_state(gst.STATE_PAUSED)
        self.paused = True
        gobject.source_remove(self.timerUpdate)

    def play(self):
        """ plays the current gst player """
        self.wTree.get_widget('preview_playButton').set_label(gtk.STOCK_MEDIA_PAUSE)
        self.player.set_state(gst.STATE_PLAYING)
        self.paused = False
        self.timerUpdater = gobject.timeout_add(500, self.callbackUpdate)

    # ----------- Signals go here ------------

    def destroy(self, widget):
        self.player.set_state(gst.STATE_READY)
        self.returnData = None
        gui.view.destroy(self, widget)



    def togglePlay(self, widget):
        """ called when the play/pause buttons are pressed """
        if self.paused == True:
            self.play()
        else:
            self.pause()



    def seek(self, widget):
        """ called when we want to seek somewhere """
        try:
            dur = self.player.query_duration(self.time_format, None)[0]
        except:
            return

        self.pause()

        position = int((dur / 100.0) * self.wTree.get_widget("preview_seekScale").get_value())

        self.player.seek_simple(self.time_format, gst.SEEK_FLAG_FLUSH, position)
        #self.player.set_state(gst.STATE_PLAYING)
        #self.paused = False
        #if not self.updateTimer:
            #self.updateTimer = gobject.timeout_add(250, self.callbackUpdate)

        self.play()


    # ----------- GST signals ------------
    def on_message(self, bus, message):

        type = message.type
        if type == gst.MESSAGE_EOS:
            self.player.set_state(gst.STATE_NULL)
            self.paused = True

        elif type == gst.MESSAGE_ERROR:
            self.player.set_state(gst.STATE_NULL)
            err, debug = message.parse_error()
            print "Error: %s" % err, debug
            self.paused = True

    def on_sync_message(self, bus, message):
        if message.structure is None:
            return

        message_name = message.structure.get_name()
        if message_name == "prepare-xwindow-id":
            imagesink = message.src
            imagesink.set_property("force-aspect-ratio", True)
            imagesink.set_xwindow_id(self.videoWidget.window.xid)


