#! /usr/bin/env python
# -*- coding: utf-8 -*-
#    This file is part of ubuntu-uploader.
#
#    ubuntu-uploader is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    ubuntu-uploader is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with ubuntu-uploader.  If not, see <http://www.gnu.org/licenses/>.
import sys, os
import pygtk
pygtk.require('2.0')
import gtk, gtk.glade, gobject
import resource, iconHandler
from resource import extVideo, extAudio, mb_it
import datetime
from math import *
import pygst
pygst.require("0.10")
import gst

from os import path as ospath
from resource import slugify
import gui

def FileChooser():
    """ opens a file chooser dialog and returns the file choosen """
    dialog = gtk.FileChooserDialog("Open..",
                                    None,
                                    gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                    gtk.STOCK_OPEN, gtk.RESPONSE_OK))
    dialog.set_default_response(gtk.RESPONSE_OK)


    response = dialog.run()
    filename = ""

    if response == gtk.RESPONSE_OK:
        filename =  dialog.get_filename()
    elif response == gtk.RESPONSE_CANCEL:
        filename = None
    dialog.destroy()

    return filename


class view(gui.view):
    """ gtk/glade object to provide the view """

    config = {
        'title': "",
        'description': "",
        'recordSound': True,
        'fps': 15,
        'quality': 90,
        'saveLocation': "~/Videos/",
    }

    done = False

    windowName = 'screencastConfigdialog'

    def __init__(self, config=None, wTree=None, callback=None):

        if not config:
            config = self.config

        self.connectSignals = {
            "on_SCCbutton_ok_clicked" : self.okayButton,

            #text entries check
            'on_SCCentry_title_changed': self.textChanged,
            'on_SCCentry_saveLocation_changed': self.textChanged,

            'on_SCCbutton_saveLocationBrowse_clicked': self.browseFolder,

            "on_SCCbutton_cancel_clicked" : self.cancelButton,
        }

        gui.view.__init__(self, config=config, wTree=wTree, callback=callback)

        #set our text buffer signals
        self.wTree.get_widget('SCCtextviewDesc').get_buffer().connect('changed', self.textChanged)

        if not self.window.is_composited():
            #we check for a composited desktop with this
            self.wTree.get_widget('SCC_warning').hide()

        self.drawConfig()


    def checkDone(self):
        """
        checks a few options in our config and returns True when they are
        in a sensible condition
        """
        if not self.wTree.get_widget('SCCentry_title').get_text():
            return False

        texbuf = self.wTree.get_widget('SCCtextviewDesc').get_buffer()
        if not texbuf.get_text(texbuf.get_start_iter(), texbuf.get_end_iter()):
            return False

        if not self.wTree.get_widget('SCCentry_saveLocation').get_text():
            return False

        return True


    def drawConfig(self):
        """ sets the current config into the windows widgets """

        title = self.config['title'] if self.config.has_key('title') else ''
        description = self.config['description'] if self.config.has_key('description') else ''
        recordSound = self.config['recordSound'] if self.config.has_key('recordSound') else False
        fps = self.config['fps'] if self.config.has_key('fps') else '8'
        quality = self.config['quality'] if self.config.has_key('quality') else '85'
        saveLoc = self.config['saveLocation'] if self.config.has_key('saveLocation') else '~/Videos/'

        saveLoc = ospath.expanduser(saveLoc)

        self.wTree.get_widget('SCCentry_title').set_text(title)

        #get our text buffer
        tbuff = self.wTree.get_widget('SCCtextviewDesc').get_buffer()
        tbuff.set_text(description)

        #set our checkbox for record sound
        self.wTree.get_widget('SCCcheckbutton_recordSound').set_active(recordSound)

        #set frames per second
        self.wTree.get_widget('SCChscale_fps').set_value(float(fps))

        #set quality
        self.wTree.get_widget('SCChscale_quality').set_value(float(quality))

        #set save location

        self.wTree.get_widget('SCCentry_saveLocation').set_text(saveLoc)
        self.config = {}


    def getConfig(self):
        """ gets the current config from the window """
        tbuf = self.wTree.get_widget('SCCtextviewDesc').get_buffer()
        self.config = {
            'title': self.wTree.get_widget('SCCentry_title').get_text(),
            'description': tbuf.get_text(tbuf.get_start_iter(), tbuf.get_end_iter()),
            'recordSound': self.wTree.get_widget('SCCcheckbutton_recordSound').get_active(),
            'fps': self.wTree.get_widget('SCChscale_fps').get_value(),
            'quality': self.wTree.get_widget('SCChscale_quality').get_value(),
            'saveLocation': self.wTree.get_widget('SCCentry_saveLocation').get_text(),
        }

        filepath = self.config['saveLocation']
        filename = slugify(self.config['title']) + ".ogg"

        #find our filename, if exists do filename(1).ogg, repeat until we have
        # a valid filename
        gotfilename = False
        counter = 1
        newname = filename
        while not gotfilename:
            if not ospath.exists(ospath.expanduser(ospath.join(filepath, newname))):
                gotfilename = True

            else:
                newname = ospath.splitext(filename)[0] + "(" + str(counter) + ")"
                newname = newname + ospath.splitext(filename)[1]
                counter += 1

        filename = newname


        self.config['filename'] = filename


    # ----- signals ------


    def textChanged(self, widget):
        """ called when the text boxes change """
        newdone = self.checkDone()
        if self.done != newdone:
            self.wTree.get_widget('SCCbutton_ok').set_sensitive(newdone)

        self.done = newdone


    def browseFolder(self, widget):
        """ called when the user clicks the browse folder button """
        chosen_dir = FileChooser()

        if chosen_dir:
            self.config['saveLocation'] = chosen_dir

        self.wTree.get_widget('SCCentry_saveLocation').set_text(chosen_dir)


