#! /usr/bin/env python
# -*- coding: utf-8 -*-
#    This file is part of ubuntu-uploader.
#
#    ubuntu-uploader is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    ubuntu-uploader is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with ubuntu-uploader.  If not, see <http://www.gnu.org/licenses/>.

import sys, os
import pygtk
pygtk.require('2.0')
import gtk, gtk.glade, gobject
import resource, iconHandler
from resource import extVideo, extAudio, mb_it
import datetime
from math import *
import pygst
pygst.require("0.10")
import gst
import subprocess
from subprocess import PIPE, call, Popen
from signal import *
import re

from os import path as ospath
from time import sleep

from random import random
from resource import slugify as fsSlugify
import gui

class rmd_process(object):
    """ object to handle record my desktop """

    percentage = 0
    progbar = None

    def setOpts(self):
        """ sets default options and options defined by config into
        self.opts
        """
        opts = []

        filepath = '~/Videos/'
        filename = 'out.ogg'

        process = None


        for key, value in self.config.iteritems():

            if key == 'fps':
                opts += ['-fps', value]

            #if key == 'title':
            #    filename = fsSlugify(value) + ".ogg"
            if key == 'filename':
                filename = value

            if key == 'recordSound' and value == False:
                opts += ['--no-sound']

            if key == 'quality':
                opts += ['-v_quality', int(63 / 100.0 * value)]
                opts += ['-s_quality', int(10 / 100.0 * value)]

            if key == 'saveLocation':
                filepath = value


        opts += ['-o', ospath.expanduser(ospath.join(filepath, filename))]

        self.opts = opts

        self.filename =  ospath.expanduser(ospath.join(filepath, filename))


    def __init__(self, config={}):
        """ pass a config to here :) """
        self.config = config
        self.opts = []
        self.setOpts()

    def startProcess(self, finishCallback):
        """ starts our recordmydesktop process """

        self.finishCallback = finishCallback

        runopts = ['recordmydesktop'] + self.opts

        # make sure we have just strings
        for index in range(len(runopts)):
            runopts[index] = str(runopts[index])


        self.process = subprocess.Popen(runopts, stdout=PIPE, stdin=PIPE, stderr=PIPE)

        #compile our regular expression
        self.regex = re.compile(r'\[(?P<percentage>\d+)\%\]')


        self.iohup = gobject.io_add_watch(self.process.stdout,
                                            gobject.IO_HUP,
                                            self.callbackIO_HUP)

        # make sure that we start our io watch
        self.ioin = gobject.io_add_watch(self.process.stdout,
                                            gobject.IO_IN,
                                            self.callbackIO_IN)


        poll = self.process.poll()
        if poll:
            raise Exception

    def poll(self):
        """ should be called every so often, checks if the process died and
        such
        """
        code = self.process.poll()

    def pauseToggle(self):
        """ causes the process to pause/unpause """
        os.kill(self.process.pid, SIGUSR1)

    def stop(self, progbar):
        """ causes the process to stop recording and start encoding """
        os.kill(self.process.pid, SIGTERM)
        self.percentage = 0
        self.progbar = progbar


    def abort(self):
        """ causes the process to stop recording and not encode a thing """
        os.kill(self.process.pid, SIGABRT)


    def readline(self, fileobj):
        """ a readline method designed to read special data from our rmd process
        essentially its able to return a carriagereturn character as a new line
        as well as \n
        """
        #data = ""
        #while True:
            #data += fileobj.read()
            #if not data:
                #return ""

            #if data[-1] == "\r":
                #break
            #if data[-1] == "\n":
                #break

        #return data

        data = fileobj.read(5)
        return data


    def callbackIO_IN(self, source, condition):
        """
        checks the stdout of the process and handles it nicely.
        actually we only need to check for two things, encoding percentage done
        and where the final file is stored
        """
        #don't remove this, its needed for some reason... fuck knows why
        sys.stdout.write("f\b")

        percentage = 0

        line = self.readline(self.process.stdout)


        try:
            percentage = int(self.regex.search(line).groupdict()['percentage'])
        except:
            return True


        while percentage < self.percentage + 1:
            line = self.readline(self.process.stdout)


            try:
                percentage = int(self.regex.search(line).groupdict()['percentage'])
            except:
                return True


        percentage = 100 if percentage >= 100 else percentage
        percentage = 0 if percentage <= 0 else percentage



        self.progbar.set_fraction(percentage / 100.0)
        self.progbar.set_text(str(percentage) + "%")

        self.percentage = percentage


        return True

    def callbackIO_HUP(self, source, condition):
        """
        simply called with the process has exited
        """
        gobject.source_remove(self.ioin)
        gobject.source_remove(self.iohup)

        self.finishCallback()

        return False




class view(gui.view):
    """ gtk/glade object to provide the view """

    config = {
        'title': "",
        'description': "",
        'recordSound': True,
        'fps': 15,
        'quality': 90,
        'saveLocation': "~/Videos/",
    }

    done = False
    statusIcon = None

    timer = None
    countdownTime = None

    process = None

    windowName = 'screencastNotification'

    def figureWindowPos(self, winGeom, dest):
        """ you pass the window geometary and the destination to this function
        and it will figure out a nice placement for it
        """

        if winGeom[0] + dest[0] > gtk.gdk.screen_width():
            #window goes off the screen on the right
            dest[0] = gtk.gdk.screen_width() - winGeom[0]


        if winGeom[1] + dest[1] > gtk.gdk.screen_height():
            #window goes off the screen on the bottom
            dest[1] = gtk.gdk.screen_height() - winGeom[1]

        if dest[0] < 0:
            dest[0] = 0

        if dest[1] < 0:
            dest[1] = 0

        return dest


    def __init__(self, wTree=None, callback=None, config=None):



        #self.window = self.wTree.get_widget("screencastNotification")
        #self.window.show()

        self.connectSignals = {
            'on_SCN_button_cancel_clicked': self.destroy,
            'on_SCN_button_pause_clicked': self.pauseCountdown,
            'on_SCN_button_record_clicked': self.startCountdown,

            #action window buttons
            'on_SCA_button_cancel_clicked': self.actionCancel,
            'on_SCA_button_stop_clicked': self.actionStop,
            'on_SCA_button_record_clicked': self.actionRecord,
        }

        gui.view.__init__(self, wTree=wTree, callback=callback, config=config)

        #create our status icon
        self.statusIcon = gtk.StatusIcon()
        self.statusIcon.set_from_stock(gtk.STOCK_MEDIA_RECORD)
        self.statusIcon.set_tooltip('Desktop Recording...')
        self.statusIcon.set_visible(True)
        self.statusIcon.connect('activate', self.statusIconActivate)

        # all this crap here is needed to position the notification
        # window properly. mainly crappy non metacity window managers this is
        # needed for *cough* compiz *cough*
        windowsize = self.window.get_size()
        screen, iconpos, orientation = self.statusIcon.get_geometry()

        windowpos = [iconpos.x, iconpos.y]

        self.figureWindowPos(windowsize, windowpos)


        #move our window close to the icon
        self.window.move(windowpos[0], windowpos[1])

        self.wTree.get_widget('SCN_button_pause').set_sensitive(False)





    def countdown_callback(self):
        """ called by the gobject countdown timer every second, decreases
        until it gets to 0, calls self.start then
        """
        self.countdownTime -= 1
        if self.countdownTime < 0:
            self.statusIcon.set_blinking(False)
            self.start()
            gobject.source_remove(self.timer)
            return False

        if self.countdownTime == 0:
            self.wTree.get_widget('SCN_button_pause').set_sensitive(False)

        if self.countdownTime > 1:
            label = 'Recording will start in %i seconds' % self.countdownTime

        elif self.countdownTime == 1:
            label = 'Recording will start in %i second' % self.countdownTime

        else:
            label = 'Recording will start now!'

        self.wTree.get_widget('SCN_label_info').set_text(label)

        return True


    def start(self):
        self.window.hide()

        #we change our window to the 'action' window
        self.window = self.wTree.get_widget("screencastActions")

        self.process = rmd_process(config = self.config)

        self.process.startProcess(self.callbackPostFinish)


    def callbackPostUpdate(self):
        """ a callback for updating the post window by checking the program's
        stdout progress
        """

        #if not self.process.percentage:
        #    return True

        percentage = self.process.percentage / 100.0
        percentage = percentage if not (percentage > 1.0) else 1.0
        percentage = percentage if not (percentage < 0.0) else 0.0

        #if percentage > 1.0:
        #    return False


        #self.wTree.get_widget('SCP_label_info').set_text(str(percentage) + "fdsfadsafsdaf")

        self.wTree.get_widget('SCP_progressbar').set_fraction(percentage)

        return True


    def callbackPostFinish(self):
        """
        called when our process has finished and exited. its assumed at this
        point everything is done :)
        """
        if self.statusIcon:
            self.statusIcon.set_visible(False)

        if self.window:
            self.window.destroy()

        if self.timer:
            gobject.source_remove(self.timer)
            self.timer = None

        self.returnData = {
            'filename': self.process.filename,
            'title': self.config['title'],
            'description': self.config['description'],
        }

        self.doCallback(senddata=True)

    # ----- signals ------
    # hello world! what a nice day it is today :)
    # lets try the stopping this thing thing... might not work...

    def destroy(self, widget):

        if self.statusIcon:
            self.statusIcon.set_visible(False)

        self.returnData = None

        gui.view.destroy(self, widget)


    def pauseCountdown(self, widget):
        gobject.source_remove(self.timer)
        self.timer = None
        self.wTree.get_widget('SCN_button_pause').set_sensitive(False)
        self.wTree.get_widget('SCN_button_record').set_sensitive(True)

        #turn off blinking
        self.statusIcon.set_blinking(False)

    def startCountdown(self, widget, seconds=8):
        """ starts a countdown, calls self.start when done """

        if self.countdownTime > 0:
            seconds = self.countdownTime


        seconds += 1

        self.countdownTime = seconds
        self.timer = gobject.timeout_add(1000, self.countdown_callback)
        self.countdown_callback()

        #set our pause button to be active, record button unactive
        self.wTree.get_widget('SCN_button_record').set_sensitive(False)
        self.wTree.get_widget('SCN_button_pause').set_sensitive(True)

        # we set our status icon to be blinking just to get the users attention
        # otherwise it *might* go unnoticed leaving the user confused
        # (some people will still be confused so at some point some
        #   notification indicating what to do should be used)

        self.statusIcon.set_blinking(True)


    def statusIconActivate(self, widget):
        """ called when the user clicks the status icon """
        self.process.pauseToggle()
        sleep(0.25)
        self.window.show()

        #position the window correctly
        windowsize = self.window.get_size()
        screen, iconpos, orientation = self.statusIcon.get_geometry()
        windowpos = [iconpos.x, iconpos.y]

        self.figureWindowPos(windowsize, windowpos)

        self.window.move(windowpos[0], windowpos[1])


    def actionCancel(self, widget):
        """ cancels recording """
        self.window.hide()
        self.process.pauseToggle()
        self.process.abort()
        self.destroy(self, widget)

    def actionStop(self, widget):
        """ stops recording """
        self.window.hide()
        sleep(0.25)
        self.process.pauseToggle()

        # setup the new 'encoding progress' window
        self.window = self.wTree.get_widget('screencastPost')
        self.window.show()

        #position the window correctly
        windowsize = self.window.get_size()
        screen, iconpos, orientation = self.statusIcon.get_geometry()
        windowpos = [iconpos.x, iconpos.y]

        self.figureWindowPos(windowsize, windowpos)

        self.window.move(windowpos[0], windowpos[1])

        # now we need to create a timer that will periodically check our
        # encoding progress and update our window accordingly

        #self.timer = gobject.timeout_add(500, self.callbackPostUpdate)

        self.process.stop(self.wTree.get_widget('SCP_progressbar'))


    def actionRecord(self, widget):
        """ starts recording again """
        self.window.hide()
        sleep(0.25)

        self.process.pauseToggle()



# HELLO WORLD! - and then some!!!
# this should all be working now. lets hope at any rate! !'s are fun!

# no they aren't...






