#!/usr/bin/env python
# -*- coding: utf-8 -*-
#       guiUploader.py
#
#       Copyright 2008 Gordon Allott <gordallott@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#
import sys, os
import pygtk
pygtk.require('2.0')
import gtk, gtk.glade, gobject
import resource, iconHandler, guiUploadConfig
from resource import extVideo, extAudio, mb_it
import datetime
from math import *
import pygst
pygst.require("0.10")
import gst

import re
from os import path as ospath
from resource import slugify
import gsocket, httplibnb
import time
import webbrowser
import gui

class average(object):
    """
    a nice pretty class for handling our runnning average of transfer rates
    """

    samples = []

    def get(self):
        """ computes an average based on our samples """
        sum = 0
        for value in self.samples:
            sum += value

        return float(sum) / len(self.samples)


    def add(self, value, maxSamples=32):
        """ adds a value to our samples, boots out old ones if needed
        """
        self.samples.append(value)
        while len(self.samples) > maxSamples:
            self.samples.pop(0)




class view(gui.view):
    """ gtk/glade object to provide the view """

    config = {
        'title': '',
        'description': '',
        'filename': '',
        'username': '',
        'password': '',
        'icon': None,
    }

    configView = None
    returnData = None

    httplib = None



    timerPulse = None
    timerProgress = None

    progressGoal = 0 #set to the total bytes wanted
    oldProgress = 0 # the last checked progress, used to calculate bytes per second
    oldTime = 0 # the time of the last check
    averageRate = average()


    #loginUrl = 'http://localhost/?q=user'
    #uploadUrl = 'http://localhost/?q=node/add/page'
    attachUrl = 'http://ubuntupeople.tv/upload/js'
    loginUrl = 'http://ubuntupeople.tv/user'
    uploadUrl = 'http://ubuntupeople.tv/node/add/page'

    windowName = 'uploader'


    def __init__(self, config = {}, wTree=None, callback=None):

        self.connectSignals = {
            "on_uploader_cancelButton_clicked" : self.cancelButton,
            'on_uploader_closeButton_clicked': self.cancelButton,
            'on_uploader_linkbutton_clicked': self.handleURL,
        }

        gui.view.__init__(self, config=config, wTree=wTree, callback=callback)

        #setup the httplib stuff
        self.reg_formtoken = re.compile(r'<input.*?name="form_token".*?value="(?P<formtoken>\w+)".*?/>')
        self.cookies = httplibnb.CookieJar()
        self.httplib = httplibnb.Http()
        self.httplib.request(self.loginUrl, self.callbackLogin)

        self.wTree.get_widget('uploader_progressbar').set_text('Contacting server')
        self.doPulse(True)

    def callbackPulse(self):
        """ a callback for giotimers that just pulses our progressbar, call
        self.doPulse(bool) to stop and start pulsing
        """
        self.wTree.get_widget('uploader_progressbar').pulse()
        return True


    def callbackProgress(self):
        """ a callback for giotimers that updates the progresbbar, it uses the
        current httplib.socket.bytes_out and compairs it to self.progressGoal
        to figure out the percentage doen
        """
        try:
            currentProgress = self.httplib.socket.bytes_out
        except:
            return False
        prog = (self.progressGoal - currentProgress) / float(self.progressGoal)
        prog = prog if prog < 1.0 else 1.0
        prog = prog if prog > 0.0 else 0.0
        prog = 1.0 - prog

        prog_text = "%s%%" % (str(round(prog, 2) * 100))
        duration = time.time() - self.oldTime
        if duration < 0.000001:
            duration = 0.0000001

        rate = (self.httplib.socket.bytes_out - self.oldProgress) / duration

        self.averageRate.add(rate)
        rate = self.averageRate.get()

        rate_text = "%sKB/s" % (str(round(rate / 1024.0,2)))

        self.wTree.get_widget('uploader_progressbar').set_fraction(prog)
        self.wTree.get_widget('uploader_progressbar').set_text(prog_text)
        self.wTree.get_widget('uploader_rate').set_text(rate_text)

        self.oldTime = time.time()
        self.oldProgress = currentProgress

        return True

    def doPulse(self, switch):
        """ turns our pulsing on or off """

        if switch:
            if self.timerProgress:
                gobject.source_remove(self.timerProgress)

            if not self.timerPulse:
                self.timerPulse = gobject.timeout_add(250, self.callbackPulse)

        else:
            if self.timerPulse:
                gobject.source_remove(self.timerPulse)


    def doProgress(self, switch, goal=1024):
        """ turns on/off updating of the progress bar """

        if switch:
            if not self.timerProgress:
                self.timerProgress = gobject.timeout_add(1000, self.callbackProgress)

            if self.timerPulse:
                gobject.source_remove(self.timerPulse)

            self.progressGoal = goal
            self.oldTime = time.time()
            self.oldProgress = 0

            #self.callbackProgress()


        else:
            if self.timerProgress:
                gobject.source_remove(self.timerProgress)
                self.progressGoal = 0
                self.oldProgress = 0




    # -------- Http callbacks --------
    def callback_postPost(self, header, body):
        #print "postpost"
        #here we just post the data
        #print body
        #self.doPulse(False)
        if header.has_key('location'):
            self.wTree.get_widget('uploader_progressbar').set_text('Success')
            self.wTree.get_widget('uploader_progressbar').set_fraction(1.0)
            self.wTree.get_widget('uploader_linkbutton').set_uri(header['location'])
            self.wTree.get_widget('uploader_linkbutton').set_label(header['location'])
            self.wTree.get_widget('uploader_linkbutton').set_sensitive(True)

            self.wTree.get_widget('uploader_cancelButton').hide()
            self.wTree.get_widget('uploader_closeButton').show()

            #print header['location']

        else:
            self.wTree.get_widget('uploader_progressbar').set_text('Failed')
            self.wTree.get_widget('uploader_progressbar').set_fraction(1.0)

        self.httplib.close()
        self.httplib = None





    def callback_midUpload(self, header, body):
        """ for the posting after the file upload """
        #print "Midupload"
        self.cookies.update(header)
        headers = {
            'Referer': self.uploadUrl,
            'Cookie': self.cookies.asString()
        }
        self.doPulse(True)
        self.wTree.get_widget('uploader_progressbar').set_text('Posting Content')
        #self.doProgress(False)


        #we need to extract our form_token from the body
        try:
            match = self.reg_formtoken.search(body)
            formtoken = match.groupdict()['formtoken']
        except:
            print "bah at formtoken"
            formtoken = ""

        filename = self.config['filepath']

        body = self.config['description'] + "\n[video]"

        form = {
            'title': (self.config['title'], 'string'),
            'changed': ('', 'string'),
            'body': (body, 'string'),
            'form_token': (formtoken, 'string'),
            'form_id': ('page_node_form', 'string'),
            'log': ('', 'string'),
            'files[upload]': ('', 'file'),
            'attach-url': (self.attachUrl, 'string'),
            'vid': ('', 'string'),
            'comment': ('0', 'string'),
            'menu[title]': ('', 'string'),
            'menu[description]': ('', 'string'),
            'menu[pid]': ('1', 'string'),
            'menu[path]': ('', 'string'),
            'menu[weight]': ('0', 'string'),
            'menu[mid]': ('0', 'string'),
            'menu[type]': ('86', 'string'),
            'name': (self.config['username'], 'string'),
            'date': ('', 'string'),
            'status': ('1', 'string'),

            'files[upload_0][list]': ('1', 'string'),
            'files[upload_0][description]': (os.path.split(filename)[1], 'string'),

            'op': ('Submit', 'string'),

        }

        self.httplib.request(self.uploadUrl, self.callback_postPost,
                        method='POST', headers=headers, multipart=True, form=form)


    def callback_preUpload(self, header, body):
        """ uploading our file here """
        ##lets try and upload a new post now huh
        #print "preupload"
        self.cookies.update(header)
        self.wTree.get_widget('uploader_progressbar').set_text('Uploading file...')

        #self.doPulse(True)


        self.cookies.update(header)
        headers = {
            'Referer': self.uploadUrl,
            'Cookie': self.cookies.asString()
        }

        #we need to extracxt our form_token from the body
        try:
            match = self.reg_formtoken.search(body)
            formtoken = match.groupdict()['formtoken']
        except:

            print "noformtoken"
            formtoken = ""

        filename = self.config['filepath']

        body = self.config['description'] + "\n[video]"

        form = {
            'title': (self.config['title'], 'string'),
            'changed': ('', 'string'),
            'body': (body, 'string'),
            'form_token': (formtoken, 'string'),
            'form_id': ('page_node_form', 'string'),
            'log': ('', 'string'),
            'files[upload]': (filename, 'file'),
            'attach-url': (self.attachUrl, 'string'),
            'vid': ('', 'string'),
            'comment': ('0', 'string'),
            'menu[title]': ('', 'string'),
            'menu[description]': ('', 'string'),
            'menu[pid]': ('1', 'string'),
            'menu[path]': ('', 'string'),
            'menu[weight]': ('0', 'string'),
            'menu[mid]': ('0', 'string'),
            'menu[type]': ('86', 'string'),
            'name': (self.config['username'], 'string'),
            'date': ('', 'string'),
            'status': ('1', 'string'),
        }


        #we add 'attach' to the form as we are uploading
        form['attach'] = ('Attach', 'string')

        outbytes = self.httplib.request(self.uploadUrl, self.callback_midUpload,
                        method='POST', headers=headers, multipart=True, form=form)

        self.doProgress(True, goal = outbytes)#os.path.getsize(self.config['filepath']))


    def callbackLoggedin(self, header, body):
        # here we should get a new 'location' in the header
        #print "logged in"
        if header.has_key('location'):
            self.wTree.get_widget('uploader_progressbar').set_text('Login Sucessful')
            self.doPulse(True)

        else:
            self.wTree.get_widget('uploader_progressbar').set_text('Login Failed')
            self.doPulse(False)
            self.wTree.get_widget('uploader_progressbar').set_fraction(1.0)

            return



        ##lets try and upload a new post now huh
        self.cookies.update(header)
        headers = {
            'Referer': self.loginUrl,
            'Cookie': self.cookies.asString()
        }

        self.httplib.request(self.uploadUrl, self.callback_preUpload,
                                method='GET', headers=headers)


    def callbackLogin(self, header, body):
        #update our progressbar
        #print "meh"
        self.wTree.get_widget('uploader_progressbar').set_text('Logging in...')
        self.doPulse(True)
        self.cookies.update(header)

        headers = {
            'Referer': self.loginUrl,
            'Cookie': self.cookies.asString()
        }
        #headers[ pushCookies(headers, cookies)
        myform = {
            'name': (self.config['username'], 'string'),
            'pass': (self.config['password'], 'string'),
            'form_id':('user_login', 'string'),
            'op':('Log in', 'string'),
        }

        self.httplib.request(self.loginUrl, self.callbackLoggedin, headers=headers,
                                    method='POST', form=myform)




    # -------- callbacks --------
    def destroy(self, widget):
        if self.httplib:
            self.httplib.close()
            self.httplib = None

        gui.view.destroy(self, widget)

    def handleURL(self, widget):

        url = widget.get_uri()
        webbrowser.open(url)


