#! /usr/bin/env python
# -*- coding: utf-8 -*-
#    This file is part of ubuntu-uploader.
#
#    ubuntu-uploader is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    ubuntu-uploader is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with ubuntu-uploader.  If not, see <http://www.gnu.org/licenses/>.

# basic idea here is that this code should be able to take a filepath and
# return a path to an appropriate icon for that filepath.
#
# this may take the form of a music icon or a screenshot of a video/image
# or whatever

import os, sys
from os import path as ospath
import gtk
from mimetypes import guess_type
import resource
from resource import extVideo, extAudio
import subprocess

def makeThumbnail(filepath):
    """
    uses *-video-thumbnailer to generate thumbnail images for use as icons
    """
    #check the filepath actually exists
    if not ospath.exists(filepath):
        print "file %s does not exist? whys that then... shouldn't really happen" % (filepath)
        raise Exception


    uuid = resource.generateUUID(filepath)

    #make sure that thumbnail folder exists
    if not ospath.exists(ospath.expanduser('~/.ubuntu-uploader/thumbnails/')):
        try:
            os.mkdir(ospath.expanduser('~/.ubuntu-uploader/thumbnails/'))
        except:
            raise Exception

    #check to see if our thumbnail already exists
    thumbnail_path = '~/.ubuntu-uploader/thumbnails/' + uuid + '.png'
    thumbnail_path = ospath.expanduser(thumbnail_path)

    if ospath.exists(thumbnail_path):
        return thumbnail_path

    #find our working thumbnailer
    binary = ""
    try:
        proc = subprocess.Popen('ffmpegthumbnailer', stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        proc.wait()
        binary = 'ffmpegthumbnailer'
    except:
        raise Exception

    opts = [binary]
    #add our opts for the ffmpegthumbnailer

    opts += ['-f']
    opts += ['-i', filepath]
    opts += ['-o', thumbnail_path]

    proc = subprocess.Popen(opts, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    proc.wait()

    #we should now have a file, lets do a check
    if not ospath.exists(thumbnail_path):
        raise Exception

    return thumbnail_path




def getIcon(filepath, size=96):
    """
    returns a tuple containing the icon type and data accoiated with it
    either:
        ("stock", GTK.stockimagething)
        ("path", "/path/to/icon.png")
        ("pixbuf", gtk.pixbuf)
    """


    #get the extension of the filename
    ext = os.path.splitext(filepath)[1]

    #get the mimetype
    mime = guess_type(ext)[0].split("/")[0] if guess_type(ext)[0] else None

    if ext[1:] in extVideo or mime == "video":
        #first we try and make a thumbnail of it
        try:
            thumbpath = makeThumbnail(filepath)
            return ('path', thumbpath)
        except:
            print "failed to generate thumbnail for %s" % (filepath)


    if ext[1:] in extAudio or mime == "audio":

        iconTheme = gtk.icon_theme_get_default()

        if  iconTheme.has_icon('audio-x-generic'):
            return ('pixbuf',
                    iconTheme.load_icon('audio-x-generic',
                                        size,
                                        0)
                    )


    if ext[1:] in extideo or mime == "video":
        # we have 'video' split in two so that file formats such as ogg get treated
        # correctly, if there is a video in there a thumbnail will be generated
        # else it will fall back and get an audio icon

        iconTheme = gtk.icon_theme_get_default()

        if  iconTheme.has_icon('video-x-generic'):
            return ('pixbuf',
                    iconTheme.load_icon('video-x-generic',
                                        size,
                                        0)
                    )



    return ("stock", gtk.STOCK_MISSING_IMAGE)
