#! /usr/bin/env python
# -*- coding: utf-8 -*-
#    This file is part of ubuntu-uploader.
#
#    ubuntu-uploader is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    ubuntu-uploader is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with ubuntu-uploader.  If not, see <http://www.gnu.org/licenses/>.

# basic xml should follow this
#   <media type="video">
#       <title>foobar</title>
#       <description>This is a foo that also sometimes bars</description>
#       <filepath>/home/foobar/Videos/foobar.avi</filepath>
#       <UUID>247d1aea-2349-5431-8624-9a72dce3f551</UUID>
#       <last_modified>1213042039.0</last_modified>
#   </media>

import os, sys
import xml.dom.minidom
import resource

class mediaLibrary(dict):
    """
        the media library stores items of interest for our media library.
        hopefully save and load at least
    """
    xmlfile = ""

    dom = None

    def __init__(self, xmlfile=None):
        """ xmlfile is the file path to the xmlfile that will be used """
        if xmlfile:
            self.xmlfile = xmlfile

        else:
            self.xmlfile = None

        self.loadXML(xmlfile)

        # init our dict methods
        dict.__init__(self)

        #parse our XML
        self.parseXML()

        self.garbageCollect()


    def garbageCollect(self):
        """ removes broken/unwanted elements """
        uuids = []
        for uuid in self:
            uuids.append(uuid)

        for uuid in uuids:
            if not os.path.exists(self[uuid]['filepath']):
                self.pop(uuid)

    def debugXML(self):
        """ just prints out the current xml """
        print self.dom.toxml()

    def parseXML(self):
        mediaEl = None
        tmpdict = {}

        #we need to get a list of our 'media' elements in the DOM
        for el in self.dom.getElementsByTagName("media"):
            MediaEl = el
            tmpdict = {}

            # now we get all the childs of our media element and add them to
            # our dictionary
            for child in el.childNodes:

                try:
                    if child.tagName:
                        #find the text data
                        data = ""
                        for text in child.childNodes:
                            if text.nodeValue.rstrip():
                                data += text.nodeValue.rstrip().lstrip()

                        tmpdict[child.tagName] = data
                except:
                    #this is needed to catch text things
                    pass

            #if we have a UUID (essential!!!!) we add it to our media library
            if tmpdict.has_key('UUID'):
                self.__setitem__(tmpdict['UUID'], tmpdict)



    def loadXML(self, xmllocation=None):
        """ loads data from an xml file """
        if xmllocation and os.path.exists(xmllocation):
            self.dom = xml.dom.minidom.parse(xmllocation)
        else:
            self.dom = xml.dom.minidom.Document()
            self.dom.appendChild(self.dom.createElement("root"))

    def saveXML(self):
        """ saves the current xml to a file """
        #xml.dom.minidom.PrettyPrint(self.dom, open(self.xmlfile, w))
        self.dicttodom()

        store = open(self.xmlfile, "w")
        store.write(self.dom.toprettyxml())
        store.close()

    def dicttodom(self):
        """ converts the current dictionary into a DOM object """
        #create a new dom, our current one needs to be replaced
        newDom = xml.dom.minidom.Document()
        newDom.appendChild(self.dom.createElement("root"))

        #find our root element
        root = newDom.getElementsByTagName("root")

        for item in self:
            #create a new element for the item, call it 'media'
            mediaEl = self.dom.createElement("media")

            for key, value in self[item].iteritems():
                newEl = self.dom.createElement(str(key))
                newEl.appendChild(self.dom.createTextNode(str(value)))

                mediaEl.appendChild(newEl)

                #append the element to the root of our DOM
                root[0].appendChild(mediaEl)

        #perform our dom swap
        self.dom = newDom


    def addMedia(self, filepath, title="", description=""):
        """ adds a 'media' file to the media library """
        if not os.path.exists(filepath):
            raise Exception

        file_uuid = str(resource.generateUUID(filepath))
        file_modified = str(os.path.getmtime(filepath))

        newdict = {}
        newdict['title'] = title
        newdict['description'] = description
        newdict['filepath'] = filepath
        newdict['UUID'] = file_uuid
        newdict['last_modified'] = file_modified

        self[file_uuid] = newdict

        return file_uuid






