#! /usr/bin/env python
# -*- coding: utf-8 -*-
#    This file is part of ubuntu-uploader.
#
#    ubuntu-uploader is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    ubuntu-uploader is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with ubuntu-uploader.  If not, see <http://www.gnu.org/licenses/>.

import os, sys
import uuid
import re
extAudio = ["wav", "ogg", "mp3", "aiff", "flac", "au", "aac", "wma", "ra",
            "ram", "mp4", "speex"]

extVideo = ["avi", "ogg", "ogm", "wmv", "mpg", "mpeg", "mov"]

def get_resource(filename):
    # !!FIXME - make this be nice and work with /usr/lib or wherever a .deb
    # or whatever installs the program to
    fullname = os.path.join('lib', filename)
    return fullname

def generateUUID(filename):
    """
    generates a UUID based on the given filename
    """
    return str(uuid.uuid5(uuid.NAMESPACE_URL, str(filename)))

def mb_it(bytes):
    """ turns an integer of bytes into a nicely formatted string indicating how
    large it is in 'human' terms
    """
    if bytes > 1024 * 1024 * 1024:
        return str(round(bytes/1024.0/1024.0/1024.0, 2)) + " GigaBytes"

    if bytes > 1024 * 1024:
        return str(round(bytes/1024.0/1024.0, 2)) + " MegaBytes"

    if bytes > 1024:
        return str(round(bytes/1024.0, 2)) + " KiloBytes"

    return str(bytes) + " Bytes"

def joinlist(inlist, char):
    """ joins a list together, puts char inbetween them """
    endstr = ''
    for item in inlist:
        endstr += str(item) + '|'
    return endstr

def slugify(s):
    """
    changes, e.g., "Petty theft" to "petty_theft"
    """
    # remove all these words from the string before urlifying
    removelist = [
        "a", "an", "as", "at", "before", "but", "by", "for", "from",
        "is", "in", "into", "like", "of", "off", "on", "onto",
        "since", "than", "the", "this", "that", "to", "up", "via",
        "with", "per"
    ]
    s = re.sub('\\b(' + joinlist(removelist, '|') + ')\\b', '', s)
    s = s.rstrip().lstrip()
    s = re.sub(r'\s+', ' ', s)
    return s
